@@ -1,8 +1,9 @@
 /*
     Silky - A GTK+ client for SILC.
-    Copyright (C) 2003,2004 Toni Willberg
+    Copyright (C) 2003, 2004, 2005 Toni Willberg
 
     - UI callbacks
+    FIXME: move all stuff from this file to other ui_ -files
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,11 +23,35 @@
 
 */
 
-#include "includes.h"
+#include <glade/glade.h>
+#include <glib.h>
+#include <silcincludes.h>
+#include <silcclient.h>
+#include <fcntl.h>
+#include <gdk/gdkkeysyms.h>
+
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include "ui_buddylist.h"
 #include "ui_channel.h"
+#include "ui_console.h"
+#include "ui_query.h"
 
+#include "callbacks.h"
+#include "support.h" /* silkyStruct() */
+#include "cmdhistory.h" /* SilkyCmdHistoryItem() */
+#include "buddylist.h" /* buddyAdd/( */
+#include "preferences.h" /* prefs_str_to_bool(), prefs_get() */
+#include "commands.h"
+#include "gui.h"
+#include "log.h"
 
+#include "servers.h" /* servers_store_to_xml() */
+#include "xmlconfig.h"
+#include "xmllayout.h"
+#include "common.h"
 
 extern GladeXML *xmlmain;
 extern silkyStruct *silky;
@@ -35,6 +60,7 @@ extern gchar *prefpath;
 extern gchar *serverspath;
 extern SilkyChannel *channels; /* from channel.c */
 extern SilkyChannel *channel_active;
+extern SilkyQuery *query_active;
 extern SilkyCmdHistoryItem *current_cmd_history_entry;	/* from cmdhistory.c */
 
 /*
@@ -44,6 +70,7 @@ extern SilkyCmdHistoryItem *current_cmd_history_entry;	/* from cmdhistory.c */
 gboolean on_command_buddy_add(GtkMenuItem *menuitem, gpointer clientid) {
   SilcClientEntry cliententry;
   gchar *fingerprint_str;
+  SilcBuffer buffer;
 
   cliententry = silc_client_get_client_by_id(silky->client, silky->conn, clientid);
   if (!cliententry) {
@@ -55,6 +82,16 @@ gboolean on_command_buddy_add(GtkMenuItem *menuitem, gpointer clientid) {
 
   fingerprint_str = silc_fingerprint(cliententry->fingerprint, cliententry->fingerprint_len);
   debug("nick: %s, fingerprint: %s", cliententry->nickname, fingerprint_str);
+
+  /* get the key */
+  debug("sending GETKEY");
+
+  buffer = silc_id_payload_encode(cliententry->id, SILC_ID_CLIENT);
+  silc_client_command_send(silky->client, silky->conn, SILC_COMMAND_GETKEY, 0, 1, 1, buffer->data, buffer->len);
+  silc_buffer_free(buffer);
+
+  //  silc_client_command_send(silky->client, silky->conn, SILC_COMMAND_GETKEY, 0, 1, 1, cliententry->id, silc_id_payload_get_len(cliententry->id) );
+
   if (buddy_add(cliententry->nickname, fingerprint_str )) {
     debug("added, now trying to save the buddy file");
 
@@ -77,12 +114,18 @@ gint on_command_whois(GtkMenuItem *menuitem, gpointer clientid) {
   SilcBuffer idp;
   SilcClientEntry cliententry;
 
-  debug("on_command_whois()");
+  debug("on_command_whois, clientid ptr '%p'", clientid);
   if (!clientid) {
     debug("Did not receive clientid in on_nicklist_menu_activate()");
     return FALSE;
   }
 
+  /* this command requires connection */
+  if (!silky->conn) {
+    debug("not connected");
+    return FALSE;
+  }
+
   cliententry = silc_client_get_client_by_id(silky->client, silky->conn, clientid);
 
   if (!cliententry) {
@@ -108,6 +151,13 @@ gint on_command_whois(GtkMenuItem *menuitem, gpointer clientid) {
   return TRUE;
 }
 
+/* sends WHOIS -pubkey pubkeyfile */
+gint on_command_whois_pubkey(GtkMenuItem *menuitem, gpointer pubkeyfile) {
+  debug("sending command: WHOIS -pubkey %s", pubkeyfile);
+  silc_client_command_call(silky->client, silky->conn, NULL, "WHOIS", "-pubkey", pubkeyfile, NULL);
+  return TRUE;
+}
+
 
 
 gint on_command_query(GtkMenuItem *menuitem, gpointer clientid) {
@@ -325,195 +375,6 @@ gint on_command_cumode(GtkMenuItem *menuitem, gpointer clientid) {
   return FALSE;
 }
 
-/* Pops up a menu over nicklist when user presses right mousebutton .
-   The menu object is created in silccallbacks.c
-   FIXME: move this to nicklist_channel.c
-*/
-gint on_nick_pressed (GtkTreeView *treeview, GdkEventButton *event) {
-  GtkTreeSelection *selection;
-  GtkTreeIter iter;
-  GtkTreeModel *model;
-  gchar *nickname;
-  SilcClientID *clientid;
-  GtkMenu *menu;
-  GtkWidget *menuitem;
-
-  debug("on_nick_pressed() button: %d", event->button);
-
-  if (event->button == 3) {
-
-
-    /* Get selected item from the nicklist */
-    selection = gtk_tree_view_get_selection(treeview);
-
-    /* popup the menu only if something was selected */
-    if (gtk_tree_selection_get_selected (selection, &model, &iter))
-      {
-	gtk_tree_model_get (model, &iter, 2, &nickname, -1);
-	gtk_tree_model_get (model, &iter, 3, &clientid, -1);
-
-	debug ("You opened menu for '%s'", nickname);
-
-	/* create the menu */
-	menu = GTK_MENU(gtk_menu_new());
-
-
-	/* ADD MENU items and signal handlers*/
-
-	/* WHOIS */
-	menuitem = gtk_menu_item_new_with_label(_("Who is"));
-	g_signal_connect(menuitem, "activate", G_CALLBACK(on_command_whois), clientid);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-	gtk_widget_show (menuitem);
-
-	/* send a message */
-	menuitem = gtk_menu_item_new_with_label(_("Send message"));
-	g_signal_connect(menuitem, "activate", G_CALLBACK(on_command_query), clientid);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-	gtk_widget_show (menuitem);
-
-	/* add to buddylist */
-	menuitem = gtk_menu_item_new_with_label(_("Add Buddy"));
-	g_signal_connect(menuitem, "activate", G_CALLBACK(on_command_buddy_add), clientid);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-	gtk_widget_show (menuitem);
-
-
-	/* a separator here */
-	menuitem = gtk_separator_menu_item_new();
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-	gtk_widget_show(menuitem);
-
-	/* cumode +o user */
-	menuitem = gtk_menu_item_new_with_label(_("Op"));
-	g_object_set_data(G_OBJECT(menuitem), "SILC_COMMAND_CUMODE", "+o"); /* op */
- 	g_signal_connect(menuitem, "activate", G_CALLBACK(on_command_cumode), clientid);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-	gtk_widget_show(menuitem);
-
-	/* cumode -o user */
-	menuitem = gtk_menu_item_new_with_label(_("Deop"));
-	g_object_set_data(G_OBJECT(menuitem), "SILC_COMMAND_CUMODE", "-o"); /* deop */
- 	g_signal_connect(menuitem, "activate", G_CALLBACK(on_command_cumode), clientid);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-	gtk_widget_show(menuitem);
-
-	/* cumode +q user */
-	menuitem = gtk_menu_item_new_with_label(_("Mute"));
-	g_object_set_data(G_OBJECT(menuitem), "SILC_COMMAND_CUMODE", "+q"); /* mute */
- 	g_signal_connect(menuitem, "activate", G_CALLBACK(on_command_cumode), clientid);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-	gtk_widget_show(menuitem);
-
-	/* cumode -q user */
-	menuitem = gtk_menu_item_new_with_label(_("Unmute"));
-	g_object_set_data(G_OBJECT(menuitem), "SILC_COMMAND_CUMODE", "-q"); /* unmute */
- 	g_signal_connect(menuitem, "activate", G_CALLBACK(on_command_cumode), clientid);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-	gtk_widget_show(menuitem);
-
-	/* a separator here */
-	menuitem = gtk_separator_menu_item_new();
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-	gtk_widget_show(menuitem);
-
-	/* kick user */
-	menuitem = gtk_menu_item_new_with_label(_("Kick"));
-	g_signal_connect(menuitem, "activate", G_CALLBACK(on_command_kick), clientid);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-	gtk_widget_show(menuitem);
-
-
-	/* kill user */
-	/* FIXME, the on_command_kill needs some work
-	menuitem = gtk_menu_item_new_with_label(_("Kill"));
-	g_signal_connect(menuitem, "activate", G_CALLBACK(on_command_kill), clientid);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-	gtk_widget_show(menuitem);
-	*/
-
-	gtk_menu_popup (menu, NULL, NULL, NULL, NULL,
-			event->button, event->time);
-      }
-  }
-
-  return FALSE;
-}
-
-/* user clicked a nick from userlist with left mousebutton.
-This function is perhaps unnecessary.
-*/
-
-void on_nicklist_selected (GtkTreeSelection *selection, gpointer data) {
-
-  GtkTreeIter iter;
-  GtkTreeModel *model;
-  gchar *nickname;
-  SilcClientID *clientid;
-
-  debug("on_nicklist_selected()");
-
-  if (gtk_tree_selection_get_selected (selection, &model, &iter))
-    {
-      gtk_tree_model_get (model, &iter, 2, &nickname, -1);
-      gtk_tree_model_get (model, &iter, 3, &clientid, -1);
-
-      debug ("You selected '%s' ", nickname);
-      g_free (nickname);
-    }
-
-  return;
-}
-
-
-
-void on_nicklist_activated (GtkTreeView *treeview, GtkTreePath *treepath, GtkTreeViewColumn *column, gpointer user_data) {
-
-  GtkTreeIter iter;
-  GtkTreeModel *model = gtk_tree_view_get_model(treeview);
-  gchar *nickname;
-  SilcClientID *clientid;
-  SilcClientEntry cliententry;
-
-  model = gtk_tree_view_get_model(treeview);
-  gtk_tree_model_get_iter(model, &iter, treepath);
-
-  debug("on_nicklist_activated()");
-  gtk_tree_model_get (model, &iter, 2, &nickname, -1);
-  gtk_tree_model_get (model, &iter, 3, &clientid, -1);
-
-  if (!clientid) {
-    debug("Did not get clientid at on_nicklist_activated");
-    return;
-  }
-
-  if(SILC_ID_CLIENT_COMPARE(clientid, silky->conn->local_entry->id)) {
-    debug("\tI won't talk to myself. Sorry.");
-    printconsole(_("You do not want to talk to yourself."));
-      return;
-  }
-
-
-  cliententry = silc_client_get_client_by_id(silky->client, silky->conn, clientid);
-  if (!cliententry) {
-    debug("Did not get cliententry at on_nicklist_activated() ");
-    return;
-  }
-
-  /* this opens new query window if no existing is
-     open for the same clientid */
-  get_query_window(clientid);
-
-  /* set focus to inputbox of the main window*/
-  gtk_widget_grab_focus(glade_xml_get_widget (xmlmain, "inputbox"));
-
-  return;
-}
-
-
-
-
-
 
 /* SERVER */
 void on_server_connect(GtkWidget *widget, gpointer user_data) {
@@ -524,7 +385,6 @@ void on_server_connect(GtkWidget *widget, gpointer user_data) {
 
 
   /* FIXME! NEED TO CLOSE PREVIOUS CHANNELS/QUERIES WHEN SERVER CHANGES! */
-  //  window = glade_xml_get_widget (xmlmain, "window_connect_server");
   window = glade_xml_get_widget (xmlmain, "dialog_connect_server");
   input = GTK_ENTRY(glade_xml_get_widget (xmlmain, "new_servername"));
   new_server = strdup(gtk_entry_get_text(input));
@@ -540,7 +400,6 @@ void on_server_connect(GtkWidget *widget, gpointer user_data) {
   }
 
   /* hide the window */
-  //  gtk_widget_hide(glade_xml_get_widget (xmlmain, "window_connect_server"));
   gtk_widget_hide(glade_xml_get_widget (xmlmain, "dialog_connect_server"));
 
 
@@ -855,7 +714,7 @@ void on_show_new_query_window(GtkWidget *widget, gpointer user_data) {
 				    G_TYPE_POINTER  /* ClientID */
 				    );
 
-    /* TARKISTA ETTEI TÄTÄ OLLUT JO OLEMASSA! */
+    /* FIXME: check that this doesn't yet exist */
     /* save pointer of this liststore for later reference */
     g_object_set_data(G_OBJECT(widget_vboxforliststore), "new_query_stored_userstore", liststore);
 
@@ -1083,8 +942,8 @@ void on_new_query_activated(GtkTreeView *treeview, GtkTreePath *treepath, GtkTre
 
 /* Join to the channel specified in Join to a channel -dialog */
 void on_join_channel(GtkWidget *widget, gpointer user_data) {
-	const gchar *channel_name;
-	SilcBuffer idp;
+  const gchar *channel_name;
+
   debug("on_join_channel()");
 
   if (!CONNECTED) {
@@ -1105,17 +964,23 @@ void on_join_channel(GtkWidget *widget, gpointer user_data) {
 
   /* hide the dialog */
   gtk_widget_hide (glade_xml_get_widget (xmlmain, "dialog_join_channel"));
+  silky_join_channel(channel_name);
+}
 
-  idp = silc_id_payload_encode(silky->conn->local_id, SILC_ID_CLIENT);
-  if (!idp) {
-    debug("Did not get idp in on_join_channel(). Can not join.");
+void on_join_channel_invited(GtkWidget *widget, gpointer chinvobj) {
+  gchar *str = g_object_get_data(G_OBJECT(glade_xml_get_widget(xmlmain, "dialog_received_invite")), "invited_channel");
+
+  debug("on_join_channel_invited()");
+
+  if( !str || !strlen(str) ) {
+    debug("Empty channel name passed in dialog, bailing out.");
     return;
   }
 
-  silc_client_command_send(silky->client, silky->conn, SILC_COMMAND_JOIN, 0, 2,
-                           1, channel_name, strlen(channel_name),
-                           2, idp->data, idp->len);
+  silky_join_channel(str);
+  g_free(str);
 
+  gtk_widget_hide(glade_xml_get_widget(xmlmain, "dialog_received_invite"));
 }
 
 /* UI */
@@ -1132,13 +997,11 @@ gboolean on_key_press(GtkWidget *widget, GdkEventKey *key, gpointer user_data) {
  GtkWidget *inputbox = glade_xml_get_widget (xmlmain, "inputbox");
  GtkWidget *topicbox = NULL;
  gchar *newtext;
- gint pagenr;
-
-
+ gint pagenr, k = key->keyval;
 
 /* if we're not connected and user hits ENTER with no text in inputbox, */
 /*   display connection dialog */
- if( DISCONNECTED && key->keyval == 65293 && (strlen( gtk_entry_get_text(GTK_ENTRY(inputbox)) ) == 0) ) { /* the ugly number is code for ENTER */
+ if( DISCONNECTED && (k == GDK_Return || k == GDK_KP_Enter) && (strlen( gtk_entry_get_text(GTK_ENTRY(inputbox)) ) == 0) ) { /* the ugly number is code for ENTER */
   debug("displaying connection dialog");
   on_show_server_connect_window( NULL, NULL );
   return(FALSE);
@@ -1154,7 +1017,7 @@ gboolean on_key_press(GtkWidget *widget, GdkEventKey *key, gpointer user_data) {
    }
  }
 
- if( key->keyval == 65362 ) {	/* up arrow */
+ if( k == GDK_Up ) {	/* up arrow */
    debug("arrow UP");
    newtext = g_strndup( (gchar *)cmd_history_prev(), 255 );
    if( newtext && g_utf8_strlen( newtext, -1 ) ) {
@@ -1165,7 +1028,7 @@ gboolean on_key_press(GtkWidget *widget, GdkEventKey *key, gpointer user_data) {
    return(TRUE);
  }
 
- if( key->keyval == 65364 ) {	/* down arrow */
+ if( k == GDK_Down ) {	/* down arrow */
    debug("arrow DOWN");
    newtext = g_strndup( (gchar *)cmd_history_next(), 255 );
    if( newtext && g_utf8_strlen( newtext, -1 ) ) {
@@ -1183,7 +1046,7 @@ gboolean on_key_press(GtkWidget *widget, GdkEventKey *key, gpointer user_data) {
 
 
  /* page up */
- if ( key->keyval == 65365 ) {
+ if ( k == GDK_Page_Up ) {
    debug("page up key");
    /* scroll the page up */
    g_signal_emit_by_name (silky_get_active_textview(), "move-cursor", GTK_MOVEMENT_PAGES, -1, FALSE);
@@ -1191,7 +1054,7 @@ gboolean on_key_press(GtkWidget *widget, GdkEventKey *key, gpointer user_data) {
  }
 
  /* page down */
- if ( key->keyval == 65366 ) {
+ if ( k == GDK_Page_Down ) {
    debug("page down key");
    /* scroll the page up */
    g_signal_emit_by_name (silky_get_active_textview(), "move-cursor", GTK_MOVEMENT_PAGES, 1, FALSE);
@@ -1199,25 +1062,25 @@ gboolean on_key_press(GtkWidget *widget, GdkEventKey *key, gpointer user_data) {
  }
 
  /* shift+home*/
- if ( key->state & GDK_SHIFT_MASK && key->keyval == 65360 ) {
+ if ( key->state & GDK_SHIFT_MASK && k == GDK_Home ) {
    debug("shift+home key");
    return (TRUE);
  }
 
  /* shift+end*/
- if ( key->state & GDK_SHIFT_MASK && key->keyval == 65367 ) {
+ if ( key->state & GDK_SHIFT_MASK && k == GDK_End ) {
    debug("shift+end key");
    return (TRUE);
  }
 
- 
+
 
  /*
     grab ALT+1..9
     and set the give page as active
 */
- if (key->state & GDK_MOD1_MASK && ( key->keyval >= '1' && key->keyval <= '9') ) {
-   pagenr = (key->keyval - 48); /* 48 = '0' */
+ if (key->state & GDK_MOD1_MASK && ( k >= '1' && k <= '9') ) {
+   pagenr = (k - 48); /* 48 = '0' */
 
    /* set active page, index starts from 0 */
    gtk_notebook_set_current_page(GTK_NOTEBOOK(glade_xml_get_widget(xmlmain, "tabs")), pagenr-1);
@@ -1228,23 +1091,48 @@ gboolean on_key_press(GtkWidget *widget, GdkEventKey *key, gpointer user_data) {
  /*
    grab ALT+left/right, and switch notebook pages
  */
- if (key->state & GDK_MOD1_MASK && key->keyval == 65361) {
+ if (key->state & GDK_MOD1_MASK && k == GDK_Left) {
    debug("alt+left");
    pagenr = gtk_notebook_get_current_page(GTK_NOTEBOOK(glade_xml_get_widget(xmlmain, "tabs")));
    if (pagenr > 0) {
      pagenr--;
      gtk_notebook_set_current_page(GTK_NOTEBOOK(glade_xml_get_widget(xmlmain, "tabs")), pagenr);
    }
-
- } else if (key->state & GDK_MOD1_MASK && key->keyval == 65363) {
+   return (TRUE);
+ }
+ if (key->state & GDK_MOD1_MASK && k == GDK_Right) {
    debug("alt+right");
    pagenr = gtk_notebook_get_current_page(GTK_NOTEBOOK(glade_xml_get_widget(xmlmain, "tabs")));
    pagenr++;
    gtk_notebook_set_current_page(GTK_NOTEBOOK(glade_xml_get_widget(xmlmain, "tabs")), pagenr);
-
-
+   return (TRUE);
  }
 
+ /* CTRL-J for join channel dialog */
+ if( key->state & GDK_CONTROL_MASK && (k == GDK_J || k == GDK_j) ) {
+   debug("CTRL+J pressed, displaying join channel dialog");
+   on_show_join_channel_window(glade_xml_get_widget(xmlmain, "dialog_join_channel"), NULL);
+   return (TRUE);
+ }
+
+ /* CTRL-N for nickchange dialog */
+ if( key->state & GDK_CONTROL_MASK && (k == GDK_N || k == GDK_n) ) {
+   debug("CTRL+N pressed, displaying nickchange dialog");
+   on_show_nick_change_window(glade_xml_get_widget(xmlmain, "dialog_nick_change"), NULL);
+   return (TRUE);
+ }
+
+ /* CTRL-W - close current tab (channel or query) */
+ if( key->state & GDK_CONTROL_MASK && (k == GDK_W || k == GDK_w) ) {
+   if( channel_active ) {
+     debug("CTRL+W pressed, leaving active channel");
+     silky_leave_channel(channel_active->channel_entry->channel_name);
+     return (TRUE);
+   } else if( query_active ) {
+     debug("CTRL+W pressed, closing active query");
+     query_remove(query_active);
+   } else debug("No active channel or query, ignoring");
+ }
 
  /* return if it was a control character/key */
  if( g_unichar_iscntrl(key->keyval) ) {
@@ -1256,7 +1144,7 @@ gboolean on_key_press(GtkWidget *widget, GdkEventKey *key, gpointer user_data) {
    /* if inputbox didn't have focus, give it to it */
    gtk_window_set_focus(GTK_WINDOW(glade_xml_get_widget (xmlmain, "window_main")), inputbox);
    gtk_editable_set_position(GTK_EDITABLE(inputbox), -1); /* focus end of the line. */
- } else if(key->keyval == 65289) { /* 65289 is code for TAB key */
+ } else if( k == GDK_Tab ) { /* 65289 is code for TAB key */
    debug("[tab] pressed");
    channel_tab_completion();
    gtk_editable_set_position(GTK_EDITABLE(inputbox), -1); /* focus end of the line. */
@@ -1361,7 +1249,7 @@ void on_ui_send(GtkWidget *widget, gpointer input)
 				     NULL, sendflags,
 					 message, strlen(message), FALSE)) {
       debug("message sent");
-      print_mime_to_channel( silky->conn->local_entry, channel->channel_name, g_strdup("text/plain"), (int)sendflags, message);
+      print_mime_to_channel( silky->conn->local_entry, channel->channel_name, 0, g_strdup("text/plain"), (int)sendflags, message);
     }
     else {
       debug("message failed");
@@ -1670,6 +1558,27 @@ gint on_toggle_channel_settings (GtkWidget *widget, gpointer userdata) {
 }
 
 
+gboolean *on_received_invite_user_info(GtkWidget *widget, SilcClientEntry client_entry) {
+  SilkyUserInfo *user = malloc(sizeof(SilkyUserInfo));
+
+  if( !user) {
+    debug("Couldn't allocate memory for SilkyUserInfo, returning.");
+    return FALSE;
+  }
+
+  debug("starting to fill SilkyUserInfo");
+  user->nickname = g_strdup_printf("%s@%s", client_entry->nickname, client_entry->server);
+  user->username = g_strdup_printf("%s@%s", client_entry->username, client_entry->hostname);
+  user->realname = g_strdup(client_entry->realname);
+  user->channels = g_strdup(_("N/A"));
+  user->fingerprint = g_strdup(silc_fingerprint(client_entry->fingerprint, 20));
+  user->attrs = NULL;	/* FIXME: request attributes */
+
+  silky_show_user_info(user);
+  silky_free_user_info(user);
+
+  return FALSE;
+}
 
 
 /* These are not yet used. FIXME: get rid of unnecessary ones! */
