@@ -1,10 +1,11 @@
 /*
     Silky - A GTK+ client for SILC.
-    Copyright (C) 2003,2004 Toni Willberg
+    Copyright (C) 2003, 2004, 2005 Toni Willberg
 
      - Channel list management functions
      - Channel member list management functions
      - Nickname TAB-completion functions
+     - Generic SILC channel functions
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,10 +25,24 @@
 
 */
 
-#include "includes.h"
-#include "ui_channel.h"
+#include <glade/glade.h>
+#include <glib.h>
+#include <gtk/gtk.h>
+#include <silcincludes.h>
+#include <silcclient.h>
+
+#include "common.h"
+#include "gui.h"
+
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
+#include "ui_channel.h" /* for refresh_gui_nicklist() */
+
+#include "support.h"
+#include "log.h"
 
-extern GladeXML *xmlmain;
 extern silkyStruct *silky;
 
 SilkyChannel *channels = NULL;
@@ -446,66 +461,11 @@ void channel_member_free_all( SilkyChannel *channel ) {
   channel->tab_member = NULL;
 }
 
-/******************** TAB COMPLETION ************************/
-void channel_tab_completion( void ) {
-    GtkEntry *inmes = GTK_ENTRY(glade_xml_get_widget( xmlmain, "inputbox" ));
-    SilkyChannelMember *m = NULL;
-    gchar *text, *ch;
-
-    if( !channel_active ) {
-	debug("Active tab is not a channel tab.");
-	return;
-    }
-
-    /* only read prefix once */
-    if( !channel_active->tab_prefix ) {
-	text = g_strdup(gtk_entry_get_text(inmes));
-	/* let's find space or start of string */
-	for( ch = g_utf8_offset_to_pointer(text, g_utf8_strlen(text, -1)); !g_unichar_isspace(g_utf8_get_char(ch)) && (ch = g_utf8_find_prev_char(text, ch)); );
-	/* now separate last "word" and put rest into prefix */
-	channel_active->tab_text = g_strndup(text, g_utf8_pointer_to_offset(text, ch));
-	if( channel_active->tab_text && g_utf8_strlen(channel_active->tab_text, -1) )
-	  channel_active->tab_prefix = g_strdup(g_utf8_next_char(ch));
-	else
-	  channel_active->tab_prefix = g_strdup(text);
-	g_free(text);		/* we do not need this anymore */
-    }
-
-    /* tab_prefix and tab_text should not be NULL for code further below */
-    if( !channel_active->tab_prefix ) channel_active->tab_prefix = g_strdup("");
-    if( !channel_active->tab_text ) channel_active->tab_text = g_strdup("");
-
-    /* make a foobar (SilkyChannelMember) with ->next pointing to start of memberlist,
-       so we can start with ->next in both cases (first tab press or consequent tab presses */
-    if( !channel_active->tab_member ) {
-	channel_active->tab_member = malloc(sizeof(SilkyChannelMember));
-	channel_active->tab_member->next = channel_active->members;
-    }
-
-    for( m = channel_active->tab_member->next; m; m = m->next ) {
-        debug("Trying nick: '%s' against prefix '%s'", CHMEMBER_NAME(m), channel_active->tab_prefix);
-        if( !g_utf8_collate(g_utf8_casefold(channel_active->tab_prefix, g_utf8_strlen(channel_active->tab_prefix, -1)), g_utf8_casefold(CHMEMBER_NAME(m), g_utf8_strlen(channel_active->tab_prefix, -1)) ) ) {
-    	    debug("Found next suitable nick: '%s'", CHMEMBER_NAME(m));
-	    channel_active->tab_member = m;
-	    if( g_utf8_strlen(channel_active->tab_text, -1) )
-	      gtk_entry_set_text(inmes, g_strdup_printf("%s %s", channel_active->tab_text, CHMEMBER_NAME(m)));
-	    else
-	      gtk_entry_set_text(inmes, g_strdup_printf("%s: ", CHMEMBER_NAME(m)));
-	    return;
-	}
-    }
-    channel_active->tab_member = NULL;
-}
-
-void channel_reset_tab_completion( void ) {
-    if( !channel_active ) return;
-
-    channel_active->tab_member = NULL;
-    channel_active->tab_prefix = NULL;
-    channel_active->tab_text = NULL;
-}
-
 
+/*
+  called when a user's nickname changes
+  informs all channels' UI about the change
+*/
 void channels_nick_change (SilcClientEntry old_entry, SilcClientID *old_client_id, SilcClientEntry new_client_entry) {
   SilcHashTableList userhtl;
   SilcChannelEntry userchannel;
@@ -631,5 +591,28 @@ void refresh_nicklist_resolved(SilcClient client,
   debug("returning...");
 }
 
-#define I_AM_CHANOP(ch)	\
-    (((get_channel_user_by_entry(ch, silky->client))->mode & SILC_CHANNEL_UMODE_CHANOP) ? TRUE : FALSE)
+void silky_join_channel(const gchar *channel_name) {
+  SilcBuffer idp = silc_id_payload_encode(silky->conn->local_id, SILC_ID_CLIENT);
+
+  if( !idp ) {
+    debug("Couldn't get idp in silc invite handler, cannot join.");
+    return;
+  }
+
+  silc_client_command_send(silky->client, silky->conn, SILC_COMMAND_JOIN, 0, 2,
+    1, channel_name, strlen(channel_name), 2, idp->data, idp->len);
+  debug("Trying to join channel '%s'.", channel_name);
+}
+
+void silky_leave_channel(const gchar *channel_name) {
+  if( !channel_name || !strlen(channel_name) ) {
+    debug("No channel name given, can not leave.");
+    return;
+  }
+
+  if( !CONNECTED ) return;
+
+  debug("Leaving channel '%s'", channel_name);
+  printconsole(g_strdup_printf(_("Leaving channel '%s'."), channel_name));
+  silc_client_command_call(silky->client, silky->conn, NULL, "LEAVE", channel_name, NULL);
+}
