@@ -1,3 +1,10 @@
+#ifndef CMDHISTORY_H__
+#define CMDHISTORY_H__
+
+#include <gtk/gtk.h>
+#include <glade/glade.h>
+
+extern GladeXML *xmlmain;
 #define CMD_HISTORY_DEPTH_MIN	0
 #define CMD_HISTORY_DEPTH_MAX	99
 
@@ -15,3 +22,5 @@ gchar *cmd_history_prev(void);
 gchar *cmd_history_next(void);
 
 void cmd_history_push(char *text);
+
+#endif
