@@ -1,6 +1,6 @@
 /*
     Silky - A GTK+ client for SILC.
-    Copyright (C) 2003,2004 Toni Willberg
+    Copyright (C) 2003, 2004, 2005 Toni Willberg
 
     - command functions, utilities
 
@@ -22,7 +22,29 @@
 
 */
 
-#include "includes.h"
+#include <glade/glade.h>
+#include <gtk/gtk.h>
+#include <silcincludes.h>
+#include <silcclient.h>
+
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
+#include "ui_console.h"
+#include "ui_query.h"
+
+#include "support.h"
+#include "commands.h"
+#include "preferences.h"
+#include "log.h"
+#include "callbacks.h"
+#include "buddylist.h"
+#include "gui.h"
+#include "xmllayout.h"
+#include "ui_channel.h" /* get_active_channel_name() FIXME, shouldn't be used */
+
+#include "common.h"
 
 extern silkyStruct *silky;
 
@@ -32,114 +54,83 @@ extern GladeXML *xmlmain;
 extern gchar *previous_nick; /* my nickname just before sending NICK command */
 extern SilkyChannel *channels;
 
-/* cmd_query callback thingy */
-void on_new_query_resolved_cmd_query(SilcClient client, SilcClientConnection conn, SilcClientEntry *clients, SilcUInt32 clients_count, void *context) {
-  int index;
 
-  debug("on_new_query_resolved_cmd_query()");
-
-  if (clients_count == 0) {
-    debug("\tDid not find anyone");
-    errordialog(_("No users found."));
-    return;
-  }
-
-  if (clients_count) {
-
-    for (index=0; index < clients_count; index++) {
-      while (!clients[index]->fingerprint) {
-	/* JUST WAIT FOR RESOLVER */
-	/* FIXME: should check that we won't wait here forever!
-	 also run gtk and also check that the widget is still open!
-	*/
-      }
-      debug("\tFound %s@%s (%s@%s) '%s' (%s)",
-	     clients[index]->nickname,
-	     clients[index]->server,
-	     clients[index]->username,
-	     clients[index]->hostname,
-	     clients[index]->realname,
-	     silc_fingerprint(clients[index]->fingerprint,
-			      clients[index]->fingerprint_len )
-	     );
-      /* This just uses the first client in the list. Should check to make sure there arn't any more, present an option? */
-
-      on_command_query(NULL,clients[0]->id);
-    }
-
-    debug("called on_command_query.");
-  }
-
-
-}
-
-/* cmd_msg callback thingy */
+/*
+   cmd_msg callback thingy
+   This is called after user does /MSG nickname
+*/
 void on_new_msg_resolved_cmd_msg(SilcClient client, SilcClientConnection conn, SilcClientEntry *clients, SilcUInt32 clients_count, void *context) {
-  gint index, value;
+  gint value;
   gchar *message;
   SilcClientID *clientid;
   gchar *str_clientid;
   SilcMessageFlags sendflags;
   SilcClientEntry cliententry;
 
+  debug("found '%d' clients", clients_count);
+
   if (clients_count == 0) {
     debug("\tDid not find anyone");
     errordialog(_("No users found."));
     return;
   }
 
-  if (clients_count) {
+  if (clients_count > 1) {
+    printconsole(_("There are many people with same nickname, message not sent. Please specify the target."));
 
-    for (index=0; index < clients_count; index++) {
-      while (!clients[index]->fingerprint) {
-	/* JUST WAIT FOR RESOLVER */
-	/* FIXME: should check that we won't wait here forever!
-	 also run gtk and also check that the widget is still open!
-	*/
-      }
+    /* FIXME: make sure this isn't illegal way */
+    on_show_new_query_window(NULL, NULL);
+
+    return;
+    /*
+      for (index=0; index < clients_count; index++) {
       debug("\tFound %s@%s (%s@%s) '%s' (%s)",
-	     clients[index]->nickname,
-	     clients[index]->server,
-	     clients[index]->username,
-	     clients[index]->hostname,
-	     clients[index]->realname,
-	     silc_fingerprint(clients[index]->fingerprint,
-			      clients[index]->fingerprint_len )
-	     );
-      /* This just uses the first client in the list. Should check to make sure there arn't any more, present an option? */
-    }
-    clientid = clients[0]->id;
-    str_clientid = silc_id_id2str(clientid, SILC_ID_CLIENT);
-
-    cliententry = silc_client_get_client_by_id(silky->client, silky->conn, clientid);
-    if (!cliententry) {
-      debug("Did not get cliententry on_send..");
-      printconsole(_("No such user."));
-      return;
-    }
-
-    message = strdup(context);
-    if (strlen(message) <= 0) {
-      debug("empty message, skipped");
-      return;
-    }
-
-    sendflags = SILC_MESSAGE_FLAG_UTF8; /* set UTF */
-    value = prefs_str_to_bool(prefs_get("sign_private_messages"));
-
-    /* whetter to SIGN message or not */
-    if (value) {
-      sendflags |= SILC_MESSAGE_FLAG_SIGNED; /* set SIGNED */
-    }
-
-    silc_client_send_private_message(silky->client, silky->conn, cliententry,
-				     sendflags, message,
-				     strlen(message), FALSE); /* force send TRUE */
-
-    debug("private message sent to %s.", cliententry->nickname);
-    printquery(clientid, silky->conn->nickname, sendflags, message);
+      clients[index]->nickname,
+      clients[index]->server,
+      clients[index]->username,
+      clients[index]->hostname,
+      clients[index]->realname,
+      silc_fingerprint(clients[index]->fingerprint,
+      clients[index]->fingerprint_len )
+      );
+      }
+    */
+    // FIXME: ask user which user to send the message to!
   }
 
+  clientid = clients[0]->id;
+
+  str_clientid = silc_id_id2str(clientid, SILC_ID_CLIENT);
+
+  cliententry = silc_client_get_client_by_id(silky->client, silky->conn, clientid);
+  if (!cliententry) {
+    debug("Did not get cliententry on_send..");
+    printconsole(_("No such user."));
+    return;
+  }
+
+  message = strdup(context);
+  if (strlen(message) <= 0) {
+    debug("empty message, skipped");
+    return;
+  }
+
+  sendflags = SILC_MESSAGE_FLAG_UTF8; /* set UTF */
+  value = prefs_str_to_bool(prefs_get("sign_private_messages"));
+
+  /* whetter to SIGN message or not */
+  if (value) {
+    sendflags |= SILC_MESSAGE_FLAG_SIGNED; /* set SIGNED */
+  }
+
+  silc_client_send_private_message(silky->client, silky->conn, cliententry,
+				   sendflags, message,
+				   strlen(message), FALSE); /* force send TRUE */
+
+  debug_info("private message sent to %s.", cliententry->nickname);
+  printquery(clientid, silky->conn->nickname, sendflags, message);
+
+
 }
 
 
@@ -249,7 +240,7 @@ cmd_me (char *tbuf, char *word[], char *word_eol[])
   SilcClientEntry cliententry;
   gint value; /* from config */
 
-  debug("cmd_me()");
+  debug_info("cmd_me()");
 
   /* Get value of the input box */
     inmes = GTK_ENTRY(glade_xml_get_widget (xmlmain, "inputbox"));
@@ -306,7 +297,7 @@ cmd_me (char *tbuf, char *word[], char *word_eol[])
 				     NULL, sendflags,
 				     message, strlen(message), FALSE);
 
-    print_mime_to_channel( silky->conn->local_entry, channel->channel_name, g_strdup("text/plain"), (int)sendflags, message);
+    print_mime_to_channel( silky->conn->local_entry, channel->channel_name, 0, g_strdup("text/plain"), (int)sendflags, message);
 
     /* Clear the textbox */
     gtk_entry_set_text(GTK_ENTRY(inmes), "");
@@ -378,7 +369,7 @@ cmd_nick (char *tbuf, char *word[], char *word_eol[])
   const gchar *new_nick;
   const gchar *old_nick;
 
-  debug("cmd_nick()");
+  debug_info("cmd_nick()");
 
   if (!CONNECTED) {
     printconsole(_("You are not connected to server."));
@@ -411,7 +402,7 @@ static int
 cmd_raw (char *tbuf, char *word[], char *word_eol[])
 {
   gchar *cmd = strdup(word_eol[2]);
-  debug("cmd_raw(%s)", cmd );
+  debug_info("cmd_raw(%s)", cmd );
 
   if (silc_client_command_call(silky->client, silky->conn, cmd)) {
     debug("cmd was success");
@@ -430,7 +421,7 @@ cmd_part (char *tbuf, char *word[], char *word_eol[])
 {
   gchar *channel_name;
   /* SilcBuffer idp;*/
-  debug("cmd_part()");
+  debug_info("cmd_part()");
 
   /* get the channel name */
   if (*word[2] && *word[2] != '*' ) { /* name given and it's not * */
@@ -467,8 +458,7 @@ static int
 cmd_join (char *tbuf, char *word[], char *word_eol[])
 {
   const gchar *channel_name;
-  SilcBuffer idp;
-  debug("cmd_join()");
+  debug_info("cmd_join()");
 
   /* get the channel name */
   channel_name = word[2];
@@ -483,15 +473,7 @@ cmd_join (char *tbuf, char *word[], char *word_eol[])
     return 0;
   }
 
-  idp = silc_id_payload_encode(silky->conn->local_id, SILC_ID_CLIENT);
-  if (!idp) {
-    debug("Did not get idp in on_join_channel(). Can not join.");
-    return 0;
-  }
-
-  silc_client_command_send(silky->client, silky->conn, SILC_COMMAND_JOIN, 0, 2,
-                           1, channel_name, strlen(channel_name),
-                           2, idp->data, idp->len);
+  silky_join_channel(channel_name);
 
   return 2;
 }
@@ -507,7 +489,7 @@ cmd_quit (char *tbuf, char *word[], char *word_eol[])
 
   *quitmsg = '\0';
 
-  debug("cmd_quit");
+  debug_info("cmd_quit");
 
   if (strlen(word[2])) {        /* no need to populate quitmsg if !firstarg */
     debug("quitting with reason '%s'", word[2]);
@@ -544,6 +526,28 @@ cmd_quit (char *tbuf, char *word[], char *word_eol[])
   return 2; /* we should never reach here */
 }
 
+/*
+  Clears the textview of active tab
+*/
+static int
+cmd_clear ()
+{
+  GtkTextView *textview = silky_get_active_textview();
+  GtkTextBuffer *textbuffer = gtk_text_view_get_buffer(textview);
+  
+  debug_info("clearning textview");
+  if (textbuffer) {
+    gtk_text_buffer_set_text(textbuffer, "", 0); /* clear the view */
+    debug("done");
+    return 1;
+  } else {
+    debug("textbuffer not found");
+    return 0;
+  }
+
+}
+
+
 /*
  Disconnects from connected server
 */
@@ -554,17 +558,17 @@ cmd_disconnect (char *tbuf, char *word[], char *word_eol[])
   gint size = sizeof(quitmsg);
 
   if (DISCONNECTED) {
-    debug ("STATE_DISCONNECTED, aborting");
+    debug_info ("STATE_DISCONNECTED, aborting");
     printconsole(_("You are not connected to server."));
     return FALSE;
   }
   if (DISCONNECTING) {
-    debug("currently disconnecting, STATE_DISCONNECTING");
+    debug_info("currently disconnecting, STATE_DISCONNECTING");
     printconsole(g_strdup_printf(_("You are already connected to '%s'."), silky->conn->remote_host) );
     return FALSE;
   }
   if ( CONNECTING ) {
-    debug("connection already in progress, STATE_CONNECTING");
+    debug_info("connection already in progress, STATE_CONNECTING");
     printconsole(_("You are currently connecting to a server, please wait."));
     return FALSE;
   }
@@ -572,7 +576,7 @@ cmd_disconnect (char *tbuf, char *word[], char *word_eol[])
 
   *quitmsg = '\0';
 
-  debug("on cmd_disconnect");
+  debug_info("on cmd_disconnect");
 
   if (strlen(word[2])) {        /* no need to populate quitmsg if !firstarg */
     if (strlen(word_eol[2]) > 128) {
@@ -608,24 +612,24 @@ cmd_server (char *tbuf, char *word[], char *word_eol[])
   char *pass;
   GtkWidget *menuwidget;
 
-  debug("cmd_server");
+  debug_info("cmd_server");
 
   /* connection ok */
   if ( CONNECTED ) {
-    debug("already connected, STATE_CONNECTED");
+    debug_info("already connected, STATE_CONNECTED");
     printconsole(g_strdup_printf(_("You are already connected to '%s'."), silky->conn->remote_host) );
     return FALSE;
   }
 
   /* just connecting */
   if ( CONNECTING ) {
-    debug("connection already in progress, STATE_CONNECTING");
+    debug_info("connection already in progress, STATE_CONNECTING");
     printconsole(_("You are already connecting to a server, please wait."));
     return FALSE;
   }
 
   if ( DISCONNECTING ) {
-    debug("currently disconnecting, STATE_DISCONNECTING");
+    debug_info("currently disconnecting, STATE_DISCONNECTING");
     printconsole(g_strdup_printf(_("You are already connected to '%s'."), silky->conn->remote_host) );
     return FALSE;
   }
@@ -668,7 +672,7 @@ cmd_server (char *tbuf, char *word[], char *word_eol[])
 				      new_server, silky);
   }
   if (ret == -1) {
-    debug("Can not connect to the server. (why?), setting STATE_DISCONNECTED");
+    debug_info("Can not connect to the server. (why?), setting STATE_DISCONNECTED");
     silky->state = STATE_DISCONNECTED;
     printconsole(_("Can not connect to the server.")); /* FIXME: why can't ?? */
     menuwidget = glade_xml_get_widget (xmlmain, "menu_server_connect");
@@ -676,7 +680,7 @@ cmd_server (char *tbuf, char *word[], char *word_eol[])
     return 0;
   }
   else {
-    debug("\tConnected to %s.", new_server);
+    debug_info("\tConnected to %s.", new_server);
     /* state and menu will be set in the callback function */
 
   }
@@ -684,42 +688,12 @@ cmd_server (char *tbuf, char *word[], char *word_eol[])
   return 2;
 }
 
-static int
-cmd_query(char *tbuf, char *word[], char *word_eol[]) {
-  const gchar *querynick;
 
-  debug("cmd_query()");
-
-  querynick = strdup(word[2]);
-
-  if (!querynick) {
-    debug("Did not get anything from input.");
-    return FALSE;
-  }
-
-  /* Let's call WHOIS first, and then the _get_clients thing...
-     Perhaps the client library will get better solution for this some day :/
-  */
-
-  debug("Calling WHOIS '%s'...", querynick);
-  silc_client_command_call(silky->client, silky->conn, NULL, "WHOIS", querynick, NULL);
-
-  debug("Setting callback for silc_client_get_clients()...");
-
-  /* ask client library to resolve this client(s), search by nick@host, we use empty host */
-  silc_client_get_clients(silky->client, silky->conn, querynick, NULL, on_new_query_resolved_cmd_query, NULL);
-
-  /* The client library will call on_new_query_resolved_cmd_query after this! */
-  return 2;
-}
 static int
 cmd_msg(char *tbuf, char *word[], char *word_eol[]) {
   gchar *msgnick;
   gchar *msg;
-
-  debug("cmd_msg() - just a stub now, returning");
-  return(0);
-
+  debug_info("cmd_msg()");
 
   msgnick = strdup(word[2]);
 
@@ -733,15 +707,12 @@ cmd_msg(char *tbuf, char *word[], char *word_eol[]) {
     debug("Did not get anything from input.");
     return FALSE;
   }
-  debug("Calling WHOIS '%s'...", msgnick);
-  silc_client_command_call(silky->client, silky->conn, NULL, "WHOIS", msgnick, NULL);
 
-  debug("Setting callback for silc_client_get_clients()...");
+  debug("setting callback... library will call on_new_query_resolved_cmd_msg() after this");
+
+  silc_client_get_clients_whois(silky->client, silky->conn, msgnick, NULL, NULL, on_new_msg_resolved_cmd_msg, msg);
 
-  /* ask client library to resolve this client(s), search by nick@host, we use empty host */
-  silc_client_get_clients(silky->client, silky->conn, msgnick, NULL, on_new_msg_resolved_cmd_msg, msg);
 
-  /* The client library will call on_new_query_resolved_cmd_query after this! */
   return 2;
 }
 
@@ -751,7 +722,7 @@ cmd_set(char *tbuf, char *word[], char *word_eol[]) {
   gchar *attr = word[2], *attr_val = word_eol[3];
   gchar *val = NULL;
 
-  debug("attr = '%s', attr_val = '%s'", attr, attr_val);
+  debug_info("attr = '%s', attr_val = '%s'", attr, attr_val);
   if ( !g_utf8_strlen(attr, -1) ) {
     /* just list all attributes and abort */
     prefs_show_to_console();
@@ -809,7 +780,7 @@ static int
 cmd_list(char *tbuf, char *word[], char *word_eol[]) {
   gint rc;
 
-  debug("cmd_list()");
+  debug_info("cmd_list()");
   silc_client_command_call(silky->client, silky->conn, NULL, "LIST", NULL);
 
   rc = TRUE;
@@ -826,7 +797,7 @@ static int
 cmd_watch(char *tbuf, char *word[], char *word_eol[]) {
   SilcBuffer idp;
 
-  debug("cmd_watch(%s %s)", word[2], word[3]);
+  debug_info("cmd_watch(%s %s)", word[2], word[3]);
 
   if ( ( !strcmp(word[2], "") || !strcmp(word[3], "") ) ) {
     debug("empty or missing parameters");
@@ -843,7 +814,7 @@ cmd_watch(char *tbuf, char *word[], char *word_eol[]) {
 
   if  (!strcmp(word[2], "-del")) {
     watch_del(word[3]);
-    debug("added");
+    debug("removed");
     return TRUE;
   }
 
@@ -859,7 +830,7 @@ static int
 cmd_getkey(char *tbuf, char *word[], char *word_eol[]) {
 	int			 rc;
 
-	debug("cmd_getkey(%s)", word[2]);
+	debug_info("cmd_getkey(%s)", word[2]);
 	if ( !strcmp(word[2], "") ) { /* fixme: safe? */
 	  rc = FALSE;
 	}
@@ -872,17 +843,25 @@ cmd_getkey(char *tbuf, char *word[], char *word_eol[]) {
 
 
 /*
-  WHOIS
+  WHOIS nickname OR WHOIS -pubkey keyfile
 */
 static int
 cmd_whois(char *tbuf, char *word[], char *word_eol[]) {
-	int			 rc;
+	int rc;
 
-	debug("cmd_whois(%s)", word[2]);
+	debug_info("cmd_whois(%s %s)", word[2], word[3]);
 	if ( !strcmp(word[2], "") ) { /* fixme: safe? */
+	  /* empty */
 	  rc = FALSE;
+	} else if ( !strcmp(word[2], "-pubkey") && strcmp(word[3], "") ) {
+	  debug("doing whois by public key file");
+	  /* -pubkey filename */
+	  silc_client_command_call(silky->client, silky->conn, NULL, "WHOIS", word[2], word[3], NULL);
+	  rc = TRUE;
 	}
 	else {
+	  /* nickname */
+	  debug("doing whois by nickname");
 	  silc_client_command_call(silky->client, silky->conn, NULL, "WHOIS", word[2], NULL);
 	  rc = TRUE;
 	}
@@ -913,9 +892,15 @@ const struct commands xc_cmds[] = {
 			 "mask from the current channel."
 			 "If they are already on the channel this doesn't kick "
 			 "them (needs chanop)")},
+
+	**** End of non-implemented commands ***************/
+
 		{"CLEAR",
 			cmd_clear, 0, 0, N_("CLEAR, "
 			"Clears the current text window")},
+
+	/************** Not Implemented. ********************
+
 		{"CLOSE",
 			cmd_close, 0, 0, N_("CLOSE, "
 			"Closes the current window/tab")},
@@ -934,8 +919,8 @@ const struct commands xc_cmds[] = {
 
 		{"HELP", cmd_help, 0, 0, 0},
 
-	/************** Not Implemented. ********************
 
+	/**************** Not Implemented. ********************
 		{"IGNORE", cmd_ignore, 0, 0,
 	 		N_("IGNORE <mask> <types..> <options..>\n"
 	 		"    mask - host mask to ignore, eg: *!*@*.aol.com\n"
@@ -984,7 +969,8 @@ const struct commands xc_cmds[] = {
 
 		{"PART", cmd_part, 1, 1, N_("PART [<channel>], "
 			"leaves the channel, by default the current one")},
-		{"QUERY", cmd_query, 1, 0, N_("QUERY <nickname>, "
+		// {"QUERY", cmd_query, 1, 0, N_("QUERY <nickname>, "
+		{"QUERY", cmd_msg, 1, 0, N_("QUERY <nickname>, "
 			"opens up a new private conversation window with someone")},
 		{"QUIT", cmd_quit, 0, 0, N_("QUIT [<reason>], "
 			"disconnects from the current server, if any and quits Silky")},
@@ -1114,7 +1100,7 @@ handle_command (char *cmd)
 	int len;
 	int ret = TRUE;
 
-	debug("cmd: '%s'", cmd);
+	debug_info("cmd: '%s'", cmd);
 
 	/* anything below MUST DEC command_level before returning */
 
@@ -1177,7 +1163,7 @@ handle_command (char *cmd)
 
 	} else {
 
-	  debug("Unknown command '%s'", word[1] );
+	  debug_info("Unknown command '%s'", word[1] );
 	  printconsole(_("No such command."));
 	}
 
