@@ -1,3 +1,6 @@
+#ifndef COMMANDS_H__
+#define COMMANDS_H__
+
 typedef int (*cmd_callback) (char *tbuf, char *word[], char *word_eol[]);
 
 struct commands
@@ -19,3 +22,6 @@ void process_data_init (char *buf, char *cmd, char *word[], char *word_eol[], in
 void notj_msg (void);
 void notc_msg (void);
 #define PDIWORDS		32
+
+
+#endif
