@@ -1,8 +1,9 @@
 /*
     Silky - A GTK+ client for SILC.
-    Copyright (C) 2003,2004 Toni Willberg
+    Copyright (C) 2003, 2004, 2005 Toni Willberg
 
     - GUI-related functions
+    FIXME: move most of stuff from here to ui_ -files
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,8 +23,35 @@
 
 */
 
+#include <glib.h>
+#include <glade/glade.h>
+#include <gtk/gtk.h>
+#include <silcincludes.h>
+#include <silcclient.h>
 
-#include "includes.h"
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
+#include "support.h" /* context_nick_event() */
+#include "gui.h"
+#include "channel.h"
+#include "query.h"
+#include "buddylist.h"
+#include "servers.h"
+#include "preferences.h" /* prefs_get() */
+#include "ui_preferences.h" /* populate_prefs_serverlist() */
+#include "callbacks.h" /* on_window_hide() */
+#include "log.h"
+#include "commands.h"
+#include "xmllayout.h"
+#include "xmlconfig.h" /* xml_save_config() */
+#include "cmdhistory.h"
+#include "tips.h"
+#include "ui_console.h"
+#include "ui_channel.h"
+
+#include "common.h"
 
 extern GladeXML *xmlmain;
 extern silkyStruct *silky;
@@ -34,98 +62,13 @@ extern SilkyQuery *query_active;
 extern SilkyCmdHistoryItem *current_cmd_history_entry;	/* from cmdhistory.c */
 extern SilkyServer *servers;
 extern SilkyBuddy *buddies;
-extern gchar *prefpath;
-
-void on_settings_change_passphrase(GtkWidget *widget, gpointer user_data) {
-	debug("on_settings_change_passphrase");
-	gtk_widget_show (glade_xml_get_widget (xmlmain, "window_setupChangePassphrase"));
-	/* I18N This is a window title */
-	gtk_window_set_title(GTK_WINDOW(glade_xml_get_widget (xmlmain, "window_setupChangePassphrase")), g_strdup_printf("Silky v%s : %s : %s", SILKY_VERSION, _("Settings"), /* I18N This is a window title */_("Change passphrase")));
-}
 
 
-/*
-  add the user provided server to the server list
-*/
-
-void on_settings_server_add(GtkWidget *widget, gpointer user_data) {
-  gchar *address;
-  gint port = 706; /* hardcoded default, could be from user input too */
-
-  address = strdup(gtk_entry_get_text (GTK_ENTRY(widget)));
-  debug("adding new server '%s:%d'", address, port);
-  if (server_add(address, port)) {
-    debug("add ok");
-  
-    /* refresh the gui list */
-    populate_prefs_serverlist();
-
-    /* clear inputbox */
-    gtk_entry_set_text (GTK_ENTRY(widget), "");
-
-  } else {
-    debug("add failed");
-  }
-
-  g_free(address);
-  
-}
-
-/*
-  remove the selected server from the server list
-*/
-
-void on_settings_server_remove(GtkWidget *widget, gpointer user_data) {
-  GtkTreeView *treeview;
-  GtkTreeSelection *selection;
-  GtkTreeModel *model;
-  GtkTreeIter iter;
-  SilkyServer *server;
-
-  gchar *host;
-  gint port;
-
-  treeview = GTK_TREE_VIEW(glade_xml_get_widget (xmlmain, "preferences_servers_treeview"));
-
-  selection = gtk_tree_view_get_selection(treeview);
-
-  /* if something was selected */
-  if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
-    gtk_tree_model_get (model, &iter, 0, &host, -1); /* first column */
-    gtk_tree_model_get (model, &iter, 1, &port, -1); /* first column */
-    debug("selected server '%s:%d' for removal", host, port);
-    server = server_find_by_hostport(host, port);
-
-    if( server_remove(server) ) {
-      debug("remove ok");
-
-      /* refresh the gui list */
-      populate_prefs_serverlist();
-     
-    } else {
-      debug("remove failed");
-    }
-
-  } else {
-    debug("nothing selected for removal");
-  }
-
-}
-
-
-/* called when user clicks on font selection button in settings */
-void on_settings_select_font(GtkWidget *widget, gpointer user_data) {
-	debug("displaying dialog window");
-	gtk_widget_show (glade_xml_get_widget (xmlmain, "window_font_selector"));
-	/* I18N This is a window title */
-	gtk_window_set_title(GTK_WINDOW(glade_xml_get_widget (xmlmain, "window_font_selector")), g_strdup_printf("Silky v%s : %s", SILKY_VERSION, _("Select font")));
-}
-
-/* called when About button is clicked on the ui */
+/* called when About button is clicked on the menu */
 void on_ui_about(GtkWidget *widget, gpointer user_data) {
-  gtk_widget_show(glade_xml_get_widget (xmlmain, "window_about"));
+  gtk_widget_show(glade_xml_get_widget (xmlmain, "dialog_about"));
   /* I18N This is a window title */
-  gtk_window_set_title(GTK_WINDOW(glade_xml_get_widget (xmlmain, "window_about")), g_strdup_printf("Silky v%s : %s", SILKY_VERSION, _("About")));
+  gtk_label_set_label( GTK_LABEL(glade_xml_get_widget (xmlmain, "about_silky_label")) , g_strdup_printf("<b>Silky %s</b>", SILKY_VERSION) );
  }
 
 
@@ -179,15 +122,18 @@ void on_ui_tabs_switch_page(GtkNotebook *notebook, GtkNotebookPage *page,
   SilkyChannel *ch = NULL;
   SilkyQuery *q = NULL;
 
+  gui_renumber_tabs();
+
   debug ("on_ui_tabs_switch_page(%d)", pagenr);
-  ch = channel_find_by_pagenr(pagenr);
+
+  ch = channel_find_by_pagenr(gtk_notebook_get_current_page(notebook));
 
   if(ch == NULL) {
     channel_active = NULL;
     debug("no channel on this tab, setting channel_active = NULL");
   }
 
-  q = query_find_by_pagenr(pagenr);
+  q = query_find_by_pagenr(gtk_notebook_get_current_page(notebook));
 
   if(q == NULL) {
     query_active = NULL;
@@ -286,97 +232,6 @@ void on_disconnect_from_server(GtkWidget *widget, gpointer user_data) {
 
 }
 
-/*
-  Set back the original string to the ui.
-  When user typed new topic, but lost focus and didn't press the button.
-*/
-
-gboolean on_topic_restore(GtkWidget *widget, gpointer user_data) {
-  GtkWidget *wid_channel_topic;
-  GtkWidget *topicbutton;
-  SilcChannelEntry channel_entry;
-  gchar *channel_name;
-
-  return FALSE; /* FIXME:
-		   see how to reorder signals, this should be called AFTER
-		   on_topic is called.
-		   DISABLED NOW.
-		*/
-
-  channel_name = get_active_channel_name();
-  channel_entry = silc_client_get_channel(silky->client, silky->conn, channel_name);
-  if (!channel_entry) {
-    debug("didn't get channel_entry!");
-    return FALSE; /* =don't let gtk proceed */
-  }
-
-  topicbutton = get_stored_channel_widget(channel_entry->channel_name, "button_change_topic");
-  if (!GTK_WIDGET_HAS_FOCUS(widget)) { /* .glade points to topic button */
-    debug("nothing to do as user pressed the button");
-    return FALSE; /* user pressed topic change button, tell gtk to go there */
-  }
-
-  debug("restoring original topic");
-
-
-
-  wid_channel_topic = get_stored_channel_widget(channel_entry->channel_name, "channeltopic");
-
-  /* restore the old topic to UI */
-  gtk_entry_set_text(GTK_ENTRY(wid_channel_topic), channel_entry->topic);
-
-  return FALSE;
-}
-
-/*
-  when user changes the topic in the GUI
-*/
-gboolean on_topic(GtkWidget *widget, gpointer user_data) {
-  const gchar *topic;
-  gchar *channel_name;
-  gboolean ret;
-  SilcChannelEntry channel_entry;
-  GtkWidget *wid_channel_topic;
-
-  debug("on_topic()");
-
-  /* figure out the active channel */
-  channel_name = get_active_channel_name();
-  channel_entry = silc_client_get_channel(silky->client, silky->conn, channel_name);
-  if (!channel_entry) {
-    debug("didn't get channel_entry!");
-    return FALSE;
-  }
-
-  wid_channel_topic = get_stored_channel_widget(channel_entry->channel_name, "channeltopic");
-  topic = gtk_entry_get_text(GTK_ENTRY(wid_channel_topic));
-
-  debug("old topic: '%s', new topic: '%s'", channel_entry->topic, topic);
-  if (channel_entry->topic && !strcmp(topic, channel_entry->topic)) {
-    debug("topic not changed");
-    wid_channel_topic = get_stored_channel_widget(channel_entry->channel_name, "channeltopic");
-    gtk_entry_set_text(GTK_ENTRY(wid_channel_topic), channel_entry->topic); /* restore original string, just to be sure */
-    return FALSE;
-  } else {
-    gtk_entry_set_text(GTK_ENTRY(wid_channel_topic), topic); /* set new string to UI, ugly but required hack */
-  }
-
-  ret = silc_client_command_call(silky->client, silky->conn, NULL, "TOPIC", channel_name, topic, NULL);
-  if (ret == FALSE) {
-    debug("\terror");
-    return FALSE;
-  }
-  else {
-    debug("\tcommand sent successfully");
-  }
-
-  /* set focus to inputbox */
-  gtk_window_set_focus(GTK_WINDOW(glade_xml_get_widget( xmlmain, "window_main" )), glade_xml_get_widget( xmlmain, "inputbox" ));
-
-  return FALSE;
-}
-
-
 
 /* when user wants to see info about the server he's connected to */
 void on_serverinfo(GtkWidget *widget, gpointer user_data) {
@@ -419,9 +274,22 @@ void on_show_join_channel_window(GtkWidget *widget, gpointer user_data) {
   gtk_widget_show (glade_xml_get_widget (xmlmain, "dialog_join_channel"));
   /* I18N This is a window title */
   gtk_window_set_title(GTK_WINDOW(glade_xml_get_widget (xmlmain, "dialog_join_channel")), g_strdup_printf("Silky v%s : %s", SILKY_VERSION, _("Join a channel")));
+  gtk_editable_select_region(GTK_EDITABLE(glade_xml_get_widget(xmlmain, "join_channel_name")), 0, -1);
 }
 
+/* Show incoming invite dialog */
+void on_show_received_invite_window(GtkWidget *widget, gpointer user_data) {
+  debug("on_show_received_invite_window()");
 
+  if (!widget) {
+    debug("No such widget!?");
+    return;
+  }
+
+  gtk_widget_show (glade_xml_get_widget (xmlmain, "dialog_received_invite"));
+  /* I18N This is a window title */
+  gtk_window_set_title(GTK_WINDOW(glade_xml_get_widget (xmlmain, "dialog_received_invite")), g_strdup_printf("Silky v%s : %s", SILKY_VERSION, _("Join channel?")));
+}
 
 
 /* Show the CHANGE NICK dialog */
@@ -433,54 +301,13 @@ void on_show_nick_change_window(GtkWidget *widget, gpointer user_data) {
   }
 
   gtk_widget_show (glade_xml_get_widget (xmlmain, "dialog_nick_change"));
+  gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xmlmain, "new_nickname")), silky->client->nickname);
+  gtk_editable_select_region(GTK_EDITABLE(glade_xml_get_widget(xmlmain, "new_nickname")), 0, -1);
   /* I18N This is a window title */
   gtk_window_set_title(GTK_WINDOW(glade_xml_get_widget (xmlmain, "dialog_nick_change")), g_strdup_printf("Silky v%s : %s", SILKY_VERSION, _("Change nickname")));
 }
 
 
-/*
-  Adds all entries to the server list on preferences-servers page
-*/
-void populate_prefs_serverlist() {
-  SilkyServer *s;
-  GtkTreeView *treeview;
-  GtkListStore *liststore;
-  GtkTreeIter iter;
-
-  debug("populating server list for preferences");
-
-  /* get and clear current treeview */
-  treeview = GTK_TREE_VIEW(glade_xml_get_widget(xmlmain, "preferences_servers_treeview"));
-
-  /* initialize treeview if it doesn't yet exist */
-  if( !gtk_tree_view_get_column(treeview, 0) ) {
-    gui_prefs_set_up_server_treeview();
-  }
-
-  liststore = GTK_LIST_STORE(gtk_tree_view_get_model(treeview));
-  gtk_list_store_clear(liststore);
-
-  /* populate the dialog's combo list with servers */
-  for( s = servers; s; s = s->next ) {
-    if(s->hostname && g_utf8_strlen(s->hostname, -1) && s->port ) {
-      debug("server: '%s:%d'", s->hostname, s->port);
-
-      gtk_list_store_append(liststore, &iter);
-
-      gtk_list_store_set(liststore, &iter, 0, s->hostname, -1); /* name */
-      gtk_list_store_set(liststore, &iter, 1, s->port, -1); /* port */
-      //gtk_list_store_set(liststore, &iter, 2, s->default, -1); /* default yes/no */
-      
-    }
-    else {
-      debug("missing hostname or port, this is weird");
-    }
-
-  }
-
-  debug("ready");
-
-}
 
 /* Show the SERVER CONNECT dialog */
 void on_show_server_connect_window(GtkWidget *widget, gpointer user_data) {
@@ -530,25 +357,6 @@ int on_window_hide(GtkWidget *widget, gpointer window) {
 
 
 
-/*
-  called when user has selected new font from the font selector dialog
-*/
-void on_settings_selected_font(GtkWidget *font_widget, gpointer user_data) {
-  GtkWidget *font_name_entry;
-  gchar *font_name;
-
-  debug("on_settings_selected_font()");
-
-  font_name = gtk_font_selection_get_font_name(GTK_FONT_SELECTION(font_widget));
-
-  font_name_entry = glade_xml_get_widget (xmlmain, "set_font");
-  gtk_entry_set_text(GTK_ENTRY(font_name_entry), font_name);
-
-  /* hide the font selector */
-  gtk_widget_hide(glade_xml_get_widget (xmlmain, "window_font_selector"));
-}
-
-
 
 /* SETTINGS */
 void on_settings(GtkWidget *widget, gpointer user_data) {
@@ -562,11 +370,10 @@ void on_settings(GtkWidget *widget, gpointer user_data) {
 
 
 void silky_gui_show_main() {
-  /* DISABLED FOR 0.5.2 release 
+
   GtkTextBuffer *tiptext = gtk_text_view_get_buffer(GTK_TEXT_VIEW(glade_xml_get_widget(xmlmain, "tipstextview")));
   gint ret = GTK_RESPONSE_OK;
   gchar *tip = NULL;
-  */
 
   debug ("Showing the main window");
   /* generate and show the main app window */
@@ -590,8 +397,6 @@ void silky_gui_show_main() {
   silky->state = STATE_DISCONNECTED;
 
  /* show the tips window */
-  /* DISABLED FOR 0.5.2 release 
-
   ret = GTK_RESPONSE_OK;
 
   if ( prefs_str_to_bool(prefs_get("show_tips")) )
@@ -609,40 +414,9 @@ void silky_gui_show_main() {
     {
       debug ("not showing tip of the day");
     }
-  */
 
 }
 
-/*
-  This is called from the tips dialog.
-
-  Reads toggle state from tips dialog,
-  sets the value to preferences dialog,
-  saves the preferences (which reads the value
-  from the prefs. dialog).
-*/
-gboolean silky_tips_toggle (GtkWidget *widget, gpointer ptr) {
-  gboolean isactive;
-  GtkWidget *prefstoggle;
-
-  isactive = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
-  prefstoggle = glade_xml_get_widget(xmlmain, "set_tips");
-
-  if (isactive) {
-    debug("set to active");
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefstoggle), TRUE);
-  } else {
-    debug("set to inactive");
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefstoggle), FALSE);
-    prefs_set("show_tips", PREF_OFF);
-  }
-
-  /* save the preferences */
-  xml_save_config(CONFIG_MAIN, prefpath);
-
-  return TRUE;
-
-}
 
 
 
@@ -924,61 +698,193 @@ void set_textview_colors(GtkTextView *widget_textview) {
 }
 
 
-/*
-  initializes server treeview in preferences dialog
-  call populate_prefs_serverlist() later to add entries
-*/
-void gui_prefs_set_up_server_treeview() {
-  GtkWidget *treeview;
-  GtkCellRenderer *renderer_text;
-  GtkCellRenderer *renderer_toggle;
-  GtkListStore *liststore;
-  GtkTreeViewColumn *column;
-
-  debug("setting columns for server list in the preferences dialog");
-  /* Set columns for the server list of the preferences dialog */
-  liststore = gtk_list_store_new (
-                          3,
-                          G_TYPE_STRING,  /* hostname */
-                          G_TYPE_INT,  /* port */
-                          G_TYPE_BOOLEAN  /* autoconnect */
-                          );
-
-  treeview = glade_xml_get_widget(xmlmain, "preferences_servers_treeview");
-
-  /* set the model for treeview, overwrites the old one if present */
-  gtk_tree_view_set_model (GTK_TREE_VIEW(treeview), GTK_TREE_MODEL(liststore) );
-
-  renderer_text = gtk_cell_renderer_text_new();
-  renderer_toggle = gtk_cell_renderer_toggle_new();
-  gtk_cell_renderer_toggle_set_radio(GTK_CELL_RENDERER_TOGGLE(renderer_toggle), TRUE); /* TRUE=radio, FALSE=checkbox */
-
-  /* hostname */
-  column = gtk_tree_view_column_new_with_attributes (_("Hostname"), renderer_text, "text", 0, NULL);
-  gtk_tree_view_append_column (GTK_TREE_VIEW(treeview), column);
-  gtk_tree_view_set_search_column (GTK_TREE_VIEW(treeview), 0); /* set sortable column */
-  gtk_tree_view_column_set_sort_column_id(column,  0);
-
-
-  /* port */
-  column = gtk_tree_view_column_new_with_attributes (_("Port"), renderer_text, "text", 1, NULL);
-  gtk_tree_view_append_column (GTK_TREE_VIEW(treeview), column);
-  gtk_tree_view_set_search_column (GTK_TREE_VIEW(treeview), 1); /* set sortable column */
-  gtk_tree_view_column_set_sort_column_id(column,  1);
-
-  /* autoconnect */
-  /* DISABLED for 0.5.2
-  column = gtk_tree_view_column_new_with_attributes (_("Connect on start"), renderer_toggle, "active", 2, NULL);
-  gtk_tree_view_append_column (GTK_TREE_VIEW(treeview), column);
-  gtk_tree_view_set_search_column (GTK_TREE_VIEW(treeview), 2);
-  gtk_tree_view_column_set_sort_column_id(column,  2);
-  */
-  
-}
-
-
 
 gboolean on_nicktree_columns_changed( GtkTreeView *tv, gpointer data ) {
-  debug("FOOOOOOOOOOOO");
+  debug("nicktree columns has changed");
   return FALSE;
 }
+
+/* Shows a MODAL error dialog */
+void errordialog(const char *errormessage) {
+  GtkWindow *mainwin;
+  GtkMessageDialog *dialog;
+
+  mainwin = GTK_WINDOW(glade_xml_get_widget (xmlmain, "window_main"));
+  dialog = GTK_MESSAGE_DIALOG(
+			      gtk_message_dialog_new (mainwin,
+						      GTK_DIALOG_DESTROY_WITH_PARENT,
+						      GTK_MESSAGE_ERROR,
+						      GTK_BUTTONS_CLOSE,
+						      errormessage)
+			      );
+  gtk_dialog_run (GTK_DIALOG (dialog));
+  gtk_widget_destroy (GTK_WIDGET(dialog));
+
+  return;
+}
+
+
+
+/* this function closes all tabs in UI except the first (console) tab */
+void close_all_tabs( void ) {
+  GtkWidget *notebook;
+  int i, n;
+
+  if( !(notebook = glade_xml_get_widget(xmlmain, "tabs")) ) {
+    debug("could not retrieve notebook, bailing out");
+    return;
+  }
+
+  if( (n = gtk_notebook_get_n_pages(GTK_NOTEBOOK(notebook))) > 1 ) {
+    debug("removing tabs left over from previous connection");
+    gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), 0);
+    for( i = n; i > 0; i-- )
+      gtk_notebook_remove_page(GTK_NOTEBOOK(notebook), i);
+  }
+}
+
+
+/* this prints given message to the ACTIVE tab */
+int printconsole (const char *str) {
+  GtkTextView *textview = NULL;
+  gchar *timestamp;
+  gchar *outstring;
+  GtkTextBuffer *buffer;
+  GtkTextIter iter; /* pointer to current location */
+  GtkNotebook *storednotebook;
+  GtkWidget *active_page;
+  gint apagenr;
+  gchar *channel_name;
+  GladeXML *stored_query_xml;
+
+  /* make sure it was utf8 */
+  if (!g_utf8_validate(str, -1, NULL)) {
+    debug("Found non-utf8 string: '%s'", str);
+    str = g_strdup("Non-utf8 string received. Can not display.");
+  }
+
+
+  timestamp = g_strdup_printf("[%s] ", utf_timestamp() );
+  outstring = g_strdup_printf("%s", str);
+
+  /* see if there's active channel or query tab first */
+  storednotebook = GTK_NOTEBOOK(glade_xml_get_widget (xmlmain, "tabs"));
+  apagenr = gtk_notebook_get_current_page(GTK_NOTEBOOK(storednotebook));
+
+  /*
+     console or other type of tab? 0 is always console
+  */
+  if (apagenr == 0) { /* console tab */
+    debug("active: CONSOLE");
+    textview = GTK_TEXT_VIEW(glade_xml_get_widget (xmlmain, "textviewwidget"));
+  }
+  else { /* other tab */
+    debug("active: other tab... looking for type...");
+    active_page = gtk_notebook_get_nth_page(storednotebook, apagenr); /* active pagenr from GTK */
+    if (!active_page) {
+      debug("no page was active, this should not happen!");
+      return FALSE;
+    }
+
+    /* test if it was a channel tab */
+    channel_name = g_object_get_data(G_OBJECT(active_page), "channel_name");
+    if (channel_name) {
+      debug("active CHANNEL");
+      textview = GTK_TEXT_VIEW(get_stored_channel_widget(channel_name, "textviewwidget"));
+    }
+    else {
+
+      /* this way we can get the textview widget without knowing the clientid */
+      stored_query_xml = glade_get_widget_tree(active_page);
+      textview = GTK_TEXT_VIEW(glade_xml_get_widget (stored_query_xml, "textviewwidget"));
+    }
+
+  }
+
+  if (!textview) {
+    debug("active NONE. no textview found!");
+    return FALSE;
+  }
+
+
+  buffer = gtk_text_view_get_buffer(textview);
+
+  /* append \n */
+  outstring = g_strdup_printf("%s\n", outstring);
+
+  /*  debug("`%s'", outstring); */
+  /* Get the end of the buffer */
+  gtk_text_buffer_get_end_iter(buffer, &iter);
+
+  /* Print the text */
+
+  /* timestamp */
+  gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, timestamp, -1, "timestamp", NULL);
+
+  /* server message */
+  gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, outstring, -1, "server_messages", NULL);
+
+  gtk_text_buffer_get_end_iter(buffer, &iter);
+  gtk_text_buffer_create_mark(buffer, "end_mark", &iter, FALSE);
+  gtk_text_view_scroll_to_mark(textview, gtk_text_buffer_get_mark(buffer, "end_mark"), 0.1, FALSE, 0,1);
+
+  g_free(timestamp);
+  g_free(outstring);
+
+  return TRUE;
+}
+
+void silky_show_user_info(SilkyUserInfo *user) {
+  gchar **uhost = g_strsplit(user->username, "@", 2);
+  gchar **nickserv = g_strsplit(user->nickname, "@", 2);
+  SilcAttributePayload attr;
+  SilcAttribute attribute;
+  char tmp[512];
+
+  gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(xmlmain, "label_userinfo_nick")), nickserv[0]);
+  gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(xmlmain, "label_userinfo_server")), nickserv[1]);
+  gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(xmlmain, "label_userinfo_user")), uhost[0]);
+  gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(xmlmain, "label_userinfo_host")), uhost[1]);
+  gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(xmlmain, "label_userinfo_realname")), user->realname);
+  gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(xmlmain, "label_userinfo_fp")), user->fingerprint);
+  gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(xmlmain, "label_userinfo_channels")), user->channels);
+
+  /* FIXME: parse attrs */
+  if(!user->attrs) {
+    debug("No attributes to parse.");
+  } else {
+    silc_dlist_start(user->attrs);
+    while ((attr = silc_dlist_get(user->attrs)) != SILC_LIST_END) {
+      attribute = silc_attribute_get_attribute(attr);
+      memset(tmp, 0, sizeof(tmp));
+
+      switch(attribute) {
+        case SILC_ATTRIBUTE_TIMEZONE:
+          if (!silc_attribute_get_object(attr, (void *)&tmp, sizeof(tmp) - 1))
+	    continue;
+	  gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(xmlmain, "label_userinfo_tz")), tmp);
+	  break;
+      }
+    }
+  }
+
+  /* show the dialog */
+  gtk_widget_show_all(glade_xml_get_widget(xmlmain, "dialog_userinfo"));
+
+  /* The struct itself is supposed to be freed outside of this function, so it can be reused if necessary. */
+}
+
+void silky_free_user_info(SilkyUserInfo *user) {
+  if( !user ) {
+    debug("SilkyUserInfo is NULL, ignoring.");
+    return;
+  }
+
+  if(user->nickname) g_free(user->nickname);
+  if(user->username) g_free(user->username);
+  if(user->realname) g_free(user->realname);
+  if(user->fingerprint) g_free(user->fingerprint);
+  if(user->channels) g_free(user->channels);
+
+  g_free(user);
+}
