@@ -3,17 +3,11 @@
 
 
 #include "channel.h"
-
-void on_settings_change_passphrase(GtkWidget *widget, gpointer user_data);
-void on_settings_server_add(GtkWidget *widget, gpointer user_data);
-void on_settings_server_remove(GtkWidget *widget, gpointer user_data);
-void on_settings_select_font(GtkWidget *widget, gpointer user_data);
+#include "support.h"
 
 
 void silky_gui_show_main();
 void silky_gui_check_channel_cmodes();
-gboolean on_topic_restore(GtkWidget *widget, gpointer user_data);
-gboolean on_topic(GtkWidget *widget, gpointer user_data);
 
 void silky_print(gint type, void *target, gchar *text, ...);
 void silky_print_channel(gint type, SilkyChannel *channel, gchar *string);
@@ -23,13 +17,14 @@ gint gui_tab_type(gint pagenr);
 void gui_renumber_tabs();
 
 void set_textview_colors(GtkTextView *widget_textview);
+void errordialog(const char *errormessage);
+void close_all_tabs( void );
+int printconsole (const char *msg);
 
-void gui_prefs_set_up_server_treeview();
+void silky_show_user_info(SilkyUserInfo *user);
+void silky_free_user_info(SilkyUserInfo *user);
 
 
-
-void populate_prefs_serverlist();
-
 typedef enum {
   MSG_PUBLIC	= 1<<0,
   MSG_PRIVATE	= 1<<1,
