@@ -22,8 +22,30 @@
 
 */
 
-#include "includes.h"
+#include <glib.h>
+#include <glade/glade.h>
+#include <gtk/gtk.h>
+#include <silcincludes.h>
+#include <silcclient.h>
+
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+#include <string.h>
+
+#include "xmlconfig.h"
+#include "xmllayout.h"
+#include "support.h"
+#include "servers.h"
+#include "keys.h"
+#include "buddylist.h"
+#include "gui.h"
+#include "preferences.h"
+#include "timer.h"
 #include "files.h"
+#include "log.h"
+#include "common.h"
+
 
 GladeXML *xmlmain;
 silkyStruct *silky;		/* this is usually used like 'silky->conn' and 'silky->client' */
@@ -79,6 +101,8 @@ silky_initialize ()
   GtkWidget *widget;
   SilcClientParams params;
 
+  debug_info("Initializing Silky");
+
   /* Number of maximum tasks the client library's scheduler can handle.
      If set to zero, the default value will be used (200). For WIN32
      systems this should be set to 64 as it is the hard limit dictated
@@ -102,23 +126,21 @@ silky_initialize ()
   silky = silc_calloc (1, sizeof (*silky));
   if (!silky)
     {
-      debug ("Could not allocate memory for client");
-      return FALSE;
+      g_error ("Could not allocate memory for client");
     }
 
   /* Create the client */
   silky->client = silc_client_alloc (&ops, &params, silky, NULL);
   if (!silky->client)
     {
-      perror ("Could not allocate SILC client entity");
-      return FALSE;
+      g_error ("Could not allocate SILC client entity");
     }
 
 #ifdef SILKY_WIN32
   /* We need to initialize network manually if running in Win32 */
   if (silc_net_win32_init () == FALSE)
     {
-      debug ("Can not initialize Win32 network.");
+      debug_error ("Can not initialize Win32 network.");
       return FALSE;
     }
 #endif
@@ -129,13 +151,13 @@ silky_initialize ()
      First we try to get stuff from user's preferences
    */
 
-  debug ("initializing the silc client library");
+  debug_info ("Initializing the SILC Client Library");
   silky->client->username = silc_get_username ();
   if (!silky->client->username)
     {
       silky->client->username = "silkyuser";
     }
-  debug ("SILC username: '%s'", silky->client->username);
+  debug_message ("SILC username: '%s'", silky->client->username);
 
   /* Set hostname */
   silky->client->hostname = silc_net_localhost ();	/* from OS */
@@ -143,10 +165,10 @@ silky_initialize ()
     {
       silky->client->hostname = "localhost";	/* last resort */
     }
-  debug ("SILC hostname: '%s'", silky->client->hostname);
+  debug_message ("SILC hostname: '%s'", silky->client->hostname);
 
 
-  debug("Loading config...");
+  debug_info("Loading your configurations");
   if( !prefs_load_config() ) return FALSE;
 
   /* Set nickname */
@@ -163,7 +185,7 @@ silky_initialize ()
       silky->client->nickname = "silkyuser";	/* last resort */
   }
 
-  debug ("SILC nickname: '%s'", silky->client->nickname);
+  debug_message ("SILC nickname: '%s'", silky->client->nickname);
 
   if ( !string || !g_utf8_strlen(string, -1) ) {
       /* we got the nickname from somewhere else than from config, so let's
@@ -171,13 +193,11 @@ silky_initialize ()
       prefs_set("nickname", silky->client->nickname);
   }
 
-
-
   /* Set realname */
   string = prefs_get("realname");
-  debug("realname: '%s'", string);
+  debug("realname from prefs: '%s'", string);
 
-  if ( !string || strlen (strlen) < 1) {		/* if unset */
+  if ( !string || strlen (string) < 1) {		/* if unset */
       silky->client->realname = silc_get_username ();	/* username from OS */
   }  else {				/* if set */
       silky->client->realname = g_strdup (string);
@@ -188,7 +208,7 @@ silky_initialize ()
       silky->client->realname = "Silky User";	/* last resort */
   }
 
-  debug ("SILC realname: '%s'", silky->client->realname);
+  debug_message ("SILC realname: '%s'", silky->client->realname);
 
   if ( !string || !g_utf8_strlen(string, -1) )
     {
@@ -206,7 +226,11 @@ silky_initialize ()
   /* init server list */
   if( !init_servers(serverspath) ) {
     debug("Could not initialize user's server list, trying site-wide list");
+#ifdef SILKY_WIN32
+    if( !init_servers(".\\data\\servers.conf") ) {
+#else
     if( !init_servers(g_strconcat(GLADEDIR, "servers.conf", NULL)) ) {
+#endif
       debug("Could not initialize site-wide server list, creating empty");
       config_servers = xml_create_config(CONFIG_SERVERS);
       xml_save_config(CONFIG_SERVERS, serverspath);
@@ -225,8 +249,7 @@ silky_initialize ()
    */
   if (!silc_client_init (silky->client))
     {
-      perror ("Could not init client entity");
-      return FALSE;
+      g_error ("Could not init client entity");
     }
 
   debug("Allocating RNG");
@@ -234,9 +257,9 @@ silky_initialize ()
   silky->rng = silc_rng_alloc();
   if (silky->rng == NULL)
   {
-    perror("Could not initialize RNG");
-    return FALSE;
+    g_error("Could not initialize RNG");
   }
+
   debug("Seeding RNG");
   silc_rng_init(silky->rng);
 
@@ -252,11 +275,10 @@ silky_initialize ()
   SilcHash hash;
   gint i;
 
-  debug("Windows does not provide enough entropy, re-seeding RNG");
+  debug_message("Windows does not provide enough entropy, re-seeding RNG");
   if (!silc_hash_register_default())
   {
-    debug("Failed to register hashes");
-    return FALSE;
+    g_error("Failed to register hashes");
   }
 
   silc_hash_alloc("sha1", &hash); /* FIXME No check for success */
@@ -300,10 +322,7 @@ silky_initialize ()
   silc_log_set_callback (SILC_LOG_ERROR, silky_silc_log_callback, NULL);
   silc_log_set_callback (SILC_LOG_FATAL, silky_silc_log_callback, NULL);
 
-
-
-
-  if (				/* if any keys are missing ... */
+  if ( /* if any keys are missing ... */
       g_file_test (pubkey, G_FILE_TEST_EXISTS) == FALSE ||
       g_file_test (privkey, G_FILE_TEST_EXISTS) == FALSE)
     {
@@ -312,9 +331,9 @@ silky_initialize ()
       debug ("showing the first run window for key creation...");
       ret = gtk_dialog_run (GTK_DIALOG(glade_xml_get_widget (xmlmain, "dialog_firstrun")));
       gtk_widget_hide (glade_xml_get_widget (xmlmain, "dialog_firstrun"));
-      /* fixme: check ret */
+
       if (ret != GTK_RESPONSE_OK) {
-	debug("user didn't press OK, returning and quitting.");
+	debug_info("User rejected the key, quitting.");
 	return FALSE;
       }
       debug("returned from the dialog, hiding it");
@@ -325,7 +344,7 @@ silky_initialize ()
       widget = glade_xml_get_widget (xmlmain, "keygen_radiobutton_automatic");
       if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(widget)))
 	{
-	  debug("manual mode, showing 'dialog_firstrun_manual'");
+	  debug_message("Manual key generation");
 
 	  /* generate the keys */
 	  while ( 1 )
@@ -355,7 +374,7 @@ silky_initialize ()
 	  gtk_widget_hide(glade_xml_get_widget (xmlmain, "dialog_firstrun_manual"));
 	}
       else {
-	debug("automatic mode, not showing 'dialog_firstrun_manual'");
+	debug_message("Autogenerating the keys");
 
 	/* generate the keys */
 	while ( 1 )
@@ -376,7 +395,6 @@ silky_initialize ()
 
 
   /* Try opening keys without passphrase */
-  //    ret = silc_load_key_pair(pubkey, privkey, "", NULL, NULL, NULL);
   debug("Trying to load the keys without passphrase");
   ret =
     silc_load_key_pair (pubkey, privkey, "", &silky->client->pkcs,
@@ -384,10 +402,11 @@ silky_initialize ()
 			&silky->client->private_key);
   if (ret == TRUE)
     {
-      debug ("No need to ask passphrase..");
+      debug_info ("User's key doesn't have a passphrase");
     }
   else
     {
+      debug_info ("Asking passphrase for user's key");
 
       while ( 1) {
 	debug ("now asking for the passphrase...");
@@ -416,7 +435,7 @@ silky_initialize ()
 	  break;
 	}
 
-	debug("failed. wrong passphrase, lopping");
+	debug_info("Incorrect passphrase, asking again");
       }
 
     }
@@ -441,10 +460,9 @@ silky_initialize ()
 gboolean
 silky_gtk_main ()
 {
-  if (silky && silky->client)
-    {
-     silc_client_run_one (silky->client);
-     }
+  if (silky && silky->client) {
+    silc_client_run_one (silky->client);
+  }
 
   return TRUE;
 }
@@ -457,7 +475,7 @@ silky_gtk_main ()
 int
 main_quit (gpointer data)
 {
-  debug ("exiting");
+  debug_info ("Quit handler called");
   save_buddies();
 
 /* fixme; remove timers! */
@@ -470,7 +488,7 @@ main_quit (gpointer data)
 
   mime_types_list_destroy ();	/* free the mime.types list */
 
-  debug ("That's all folks!");
+  debug_message ("Quitting, bye!");
 
   xmlFreeDoc (config_main);
   xmlFreeDoc (config_servers);
@@ -489,20 +507,21 @@ main_quit (gpointer data)
 int
 main (int argc, char *argv[])
 {
-  int c;
+  int c, option_index;
+  gint optarg_int;
   static struct option long_options[] = {
     {"debug", 0, 0, 0},
+    {"debug-level", 1, 0, 0},
     {0, 0, 0, 0}
   };
 
-
+  debug_level = LOG_MESSAGE;
   gtk_init (&argc, &argv);
   glade_gnome_init ();
 
-
   /* get options */
   while (1) {
-    int option_index = 0;
+    option_index = 0;
     c = getopt_long (argc, argv, "dhv", long_options, &option_index);
 
     if (c == -1) {
@@ -511,15 +530,31 @@ main (int argc, char *argv[])
 
     switch (c) {
     case 0:
+      debug_message ("long option '%s'", long_options[option_index].name);
+      if (optarg)
+	debug_message  (" with arg '%s'", optarg);
+
       if (option_index == 0) { /* debug */
 	debug_on = 1;
-	debug("Debug turned on");
+	debug_message  ("* debugging turned on");
       }
+      if (option_index == 1) {
+	if (!debug_on) { /* could be turned on earlier */
+	  debug_on = 1;
+	  debug_message ("* debugging turned on");
+	}
+	optarg_int = strtol(optarg, NULL, 10);
+	debug_message  ("* debug_level = '%d'", optarg_int);
+	debug_level = optarg_int;
+      }
+
       break;
 
     case 'd':
-      debug_on = 1;
-      debug("Debug turned on");
+      if (!debug_on) { /* could be turned on earlier */
+	debug_on = 1;
+	debug_message ("* debugging turned on");
+      }
       break;
 
     case 'v':
@@ -529,13 +564,18 @@ main (int argc, char *argv[])
 
     case 'h':
     default:
-      printf ("Usage: %s [-h][-v][-d|--debug]\n", argv[0] );
+      printf (_("Usage: %s [-h][-v][-d][--debug-level=N]\n"), argv[0] );
+      printf (_("\t-h : Shows this help page\n"));
+      printf (_("\t-v : Shows the version number\n"));
+      printf (_("\t-d : Enables debugging with level 0\n"));
+      printf (_("\t--debug-level=N : Enables debugging with level N\n"));
+      printf (_("\tSupported debug levels: 0,1 and 2\n"));
+      printf (_("\tError messages are always printed, regardless of debug options.\n"));
       return -1;
     }
 
   }
 
-
   /* this prints all "left over" parameters
   if (optind < argc) {
     printf ("non-option ARGV-elements: ");
@@ -560,7 +600,7 @@ main (int argc, char *argv[])
     }
   else
     {
-      g_error ("Silky needs threads!\n");
+      g_error ("Silky needs threads, your system doesn't support threads.\n");
     }
 
 
@@ -600,17 +640,16 @@ main (int argc, char *argv[])
 
 
 #ifdef ENABLE_NLS
+  debug_info("Translations enabled");
   setlocale (LC_ALL, "");
   bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 #endif
 
-  if (!silky_read_mimetable ())
-    {
-      g_error ("Can not find mime.types file. Can not continue.\n");
-      /* quits here */
-    }
+  if (!silky_read_mimetable ()) {
+    g_error ("Can not find mime.types file. Can not continue.\n");
+  }
 
   /* set paths for glade files */
   glade_silky = g_strdup ("silky.glade");
@@ -618,8 +657,7 @@ main (int argc, char *argv[])
   glade_silky_query = g_strdup ("silky-query.glade");
 
   /* first try from local dir, then under GLADEDIR (Makefile.am) */
-  if (g_file_test (glade_silky, G_FILE_TEST_EXISTS) == FALSE)
-    {
+  if (g_file_test (glade_silky, G_FILE_TEST_EXISTS) == FALSE) {
       debug ("glade files not found in current dir...");
       g_free (glade_silky);
       g_free (glade_silky_channel);
@@ -628,11 +666,9 @@ main (int argc, char *argv[])
       glade_silky_channel = g_strdup (GLADEDIR "silky-channel.glade");
       glade_silky_query = g_strdup (GLADEDIR "silky-query.glade");
       debug ("using files from installation directory");
-    }
-  else
-    {
-      debug ("glade files were found in current dir");
-    }
+  } else {
+    debug ("glade files were found in current dir");
+  }
 
 
   /* add quit handler, gets called after GTK quits */
@@ -643,30 +679,22 @@ main (int argc, char *argv[])
      "xmlmain" can be used after these lines everywhere
    */
   xmlmain = glade_xml_new (glade_silky, NULL, NULL);
-  if (!xmlmain)
-    {
-      g_error ("Can not load glade file '%s'\n", glade_silky);
-    }
-  else
-    {
-      debug ("loaded .glade file '%s'", glade_silky);
-    }
+  if (!xmlmain) {
+    g_error ("Can not load glade file '%s'\n", glade_silky);
+  } else {
+    debug_info ("Loaded .glade file '%s'", glade_silky);
+  }
 
   debug ("scheduling silky_gtk_main");
 
-  if (!g_timeout_add (200, silky_gtk_main, NULL))
-    {				/* in timer.c */
-      g_error ("wasn't able to schedule silky_gtk_main\n");
-      /* quitting here */
-    }
-
+  if (!g_timeout_add (200, silky_gtk_main, NULL)) { /* in timer.c */
+    g_error ("wasn't able to schedule silky_gtk_main\n");
+  }
 
   debug("calling silky_initialize ()");
-  if (!silky_initialize ())
-    {
-      debug("Can not initialize Silky. Can not continue.");
-      return -1;
-    }
+  if (!silky_initialize ()) {
+    g_error("Can not initialize Silky. Can not continue.");
+  }
 
 
   debug ("connecting gtk signals");
@@ -681,18 +709,13 @@ main (int argc, char *argv[])
       /* quitting here */
     }
 
-
-
   silky_gui_show_main();
 
   /* give control to GTK */
-  debug ("starting gtk_main");
+  debug_info ("Starting the main loop");
 
-  // gdk_threads_enter();
   gtk_main();
-  //  gdk_threads_leave();
 
   /* we never reach this point, see _quit function */
-
   return 1;
 }
