@@ -1,4 +1,6 @@
-void prefs_fill_colormenu(GtkWidget *widget, gchar *set_color);
+#ifndef PREFERENCES_H__
+#define PREFERENCES_H__
+
 gboolean prefs_load_config();
 void prefs_populate_gui();
 void prefs_save_gui();
@@ -15,9 +17,12 @@ typedef enum
 {
  WIDGET_ENTRY,
  WIDGET_CHECKBUTTON,
- WIDGET_COLORMENU,
+ WIDGET_COLORBUTTON,
+ WIDGET_FONTBUTTON,
  WIDGET_SPINBUTTON,
 } PrefWidgetTypes;
 
 #define PREF_ON		"ON"
 #define PREF_OFF	"OFF"
+
+#endif
