@@ -1,3 +1,7 @@
+#ifndef SERVERS_H__
+#define SERVERS_H__
+
+
 struct server_t {
   struct server_t 	*next;
   gchar			*hostname;	/* the hostname of a server */
@@ -16,3 +20,6 @@ SilkyServer *server_add(gchar *host, gint port);
 gboolean server_remove( SilkyServer *server );
 
 gboolean servers_store_to_xml();
+
+
+#endif
