@@ -22,11 +22,22 @@
 
 */
 
+#include <glib.h>
+#include <glade/glade.h>
+
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
 
-#include "includes.h"
 #include "ui_buddylist.h"
+#include "channel.h"
 #include "ui_channel.h"
-
+#include "ui_console.h"
+#include "support.h"
+#include "gui.h"
+#include "log.h"
+#include "keys.h"
+#include "common.h"
 
 extern GladeXML *xmlmain;
 extern silkyStruct *silky;                 /* the client */
@@ -62,6 +73,8 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
     *str_storid, *str_oldid, *channel_usercount, *str,
     *str_fingerprint, *realname;
 
+  gchar chanstr[512];
+
   va_list va;
 
   gboolean valid;
@@ -74,7 +87,7 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
   SilcIdType entry_type;
   SilcPublicKey public_key;
 
-  SilcBuffer client_id_list, channels;
+  SilcBuffer client_id_list, channels, user_modes;
   SilcHashTableList userhtl;
 
   GtkWidget *window_channel, *widget_channel,
@@ -92,13 +105,18 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
   GtkCellRenderer *renderer_text;
   GtkWidget *widget;
   GtkTreeModel *sorted_nicklist;
-
+  SilkyBuddy *b = NULL;
   GladeXML *xml_channel;
+  SilcDList list;
 
   SilkyChannel *ch = NULL;
 
+  SilkyUserInfo *user = malloc(sizeof(SilkyUserInfo));
+
   unsigned char *fingerprint;
 
+  lastpong = time(NULL); /* update date of last received data, used in timer.c */
+
   debug("debug silc_command_reply with status (%d:%s)", status, silc_get_status_message(status));
   va_start(va, status);
 
@@ -106,11 +124,11 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
 
 
   case SILC_COMMAND_GETKEY: /* 26 */
-    debug("reply for SILC_COMMAND_GETKEY");
+    debug_info("reply for SILC_COMMAND_GETKEY");
 
     if (!success) {
 
-      debug("was not success (status %s)", silc_get_status_message(status) );
+      debug_info("was not success (status %s)", silc_get_status_message(status) );
       switch(status) {
       case SILC_STATUS_ERR_NO_SUCH_NICK:
 	printconsole(_("No such user."));
@@ -123,7 +141,7 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
       default:
 	break;
       }
-      
+
       return;
     }
 
@@ -134,8 +152,7 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
 
     {
       SilcUInt32 keysize;
-      unsigned char *encpk;
-      encpk = silc_pkcs_public_key_encode(public_key, &keysize);    
+      unsigned char *encpk = silc_pkcs_public_key_encode(public_key, &keysize);
       gchar *fingerprint = silc_hash_fingerprint(NULL, encpk, keysize);
 
       /* FIXME: use the key type (server/client) from the callback instead of hard-code
@@ -143,8 +160,8 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
       */
       if (encpk) {
 	debug("trying to save the key type '%d' '%s'...", SILC_SOCKET_TYPE_CLIENT, fingerprint);
-	silc_save_public_key(silky->client, silky->conn, 
-			     fingerprint, SILC_SOCKET_TYPE_CLIENT, 
+	silky_save_public_key(silky->client, silky->conn,
+			     fingerprint, SILC_SOCKET_TYPE_CLIENT,
 			     encpk, keysize,
 			     SILC_SKE_PK_TYPE_SILC,
 			     NULL, NULL);
@@ -158,10 +175,10 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
     break;
 
   case SILC_COMMAND_WHOIS:
-    debug("SILC_COMMAND_WHOIS");
+    debug_info("SILC_COMMAND_WHOIS");
     if (!success) {
 
-      debug("was not success (status %s)", silc_get_status_message(status) );
+      debug_info("was not success (status %s)", silc_get_status_message(status) );
 
       switch(status) {
       case SILC_STATUS_ERR_NO_SUCH_NICK:
@@ -187,16 +204,14 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
     usermodei =  va_arg(va, SilcUInt32);
     idletime =  va_arg(va, SilcUInt32);
     fingerprint = va_arg(va, unsigned char *);
-    (void)va_arg(va, SilcBuffer); /* channel_usermodes */
-    (void)va_arg(va, SilcDList); /* attrs */
+    user_modes = va_arg(va, SilcBuffer);
+    user->attrs = va_arg(va, SilcDList);
 
     if (!client_entry) {
       debug("didn't find client_entry");
       return;
     }
 
-    debug("got WHOIS for '%s'", nickname);
-
     if (fingerprint != NULL) {
       str_fingerprint = silc_fingerprint(fingerprint, 20 ); /* FIXME: is 20 okey? */
     }
@@ -205,20 +220,72 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
       str_fingerprint = g_strdup(_("N/A"));
     }
 
+    if( channels && user_modes ) {
+      SilcUInt32 *umodes;
+      list = silc_channel_payload_parse_list(channels->data, channels->len);
+      if( list && silc_get_mode_list(user_modes, silc_dlist_count(list), &umodes) ) {
+        SilcChannelPayload entry;
+	int i = 0;
+
+	memset(chanstr, 0, sizeof(chanstr));
+	silc_dlist_start(list);
+	while(( entry = silc_dlist_get(list)) != SILC_LIST_END) {
+	  SilcUInt32 name_len;
+	  char *m = silc_client_chumode_char(umodes[i++]);
+	  char *name = silc_channel_get_name(entry, &name_len);
+	  if( m ) silc_strncat(chanstr, sizeof(chanstr) - 1, m, strlen(m));
+	  silc_strncat(chanstr, sizeof(chanstr) - 1, name, name_len);
+	  silc_strncat(chanstr, sizeof(chanstr) - 1, " ", 1);
+	  silc_free(m);
+	}
+	silc_channel_payload_list_free(list);
+	silc_free(umodes);
+      }
+    }
+
+
+    /* Fill the SilkyUserInfo struct for GUI userinfo dialog */
+    user->nickname = g_strdup(nickname);
+    user->username = g_strdup(username);
+    user->realname = g_strdup(realname);
+    user->channels = g_strdup(chanstr);
+    user->fingerprint = g_strdup(str_fingerprint);
+
+    silky_show_user_info(user);
+    silky_free_user_info(user);
+
+    debug_info("got WHOIS for '%s' fingerprint: %s", nickname, str_fingerprint );
+
+    b = buddy_find_by_fp(str_fingerprint);
+    if (b) {
+      debug_info("this user is in our buddylist, updating data");
+      buddy_set_nick_by_fp(str_fingerprint, nickname);
+      buddy_set_realname_by_fp(str_fingerprint, realname);
+      buddy_set_clientid_by_fp(str_fingerprint, client_entry->id);
+      /* FIXME: parse and add user attrs if present */
+    } else {
+      debug_info("this is isn't in our buddylist");
+    }
+
+
+
+    valid = FALSE;
+
     printconsole(g_strdup_printf("%s: %s (%s)", _("Nickname"), client_entry->nickname, username));
     printconsole(g_strdup_printf("%s: %s", _("Realname"), realname));
     printconsole(g_strdup_printf("%s: %s", _("Fingerprint"), str_fingerprint));
+    printconsole(g_strdup_printf("%s: %s", _("Channels"), chanstr));
 
     g_free(str_fingerprint);
 
     break;
 
   case SILC_COMMAND_NICK:
-    debug("SILC_COMMAND_NICK");
+    debug_info("SILC_COMMAND_NICK");
 
     /* ERROR */
     if (!success) {
-      debug("Was not able to change nickname (%d)", status);
+      debug_info("Was not able to change nickname (%d)", status);
 
       if (status == SILC_STATUS_ERR_BAD_NICKNAME) {
 	printconsole(_("Illegal nickname."));
@@ -235,7 +302,7 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
     nickname = va_arg(va, char *);
     client_id = va_arg(va, SilcClientID *);
 
-    debug("I changed nick from '%s' to '%s'.", previous_nick, nickname);
+    debug_info("I changed nick from '%s' to '%s'.", previous_nick, nickname);
     printconsole( g_strdup_printf(_("Your nickname is now '%s'."), nickname));
 
     /* set my new nick entry into the label left of input line */
@@ -290,9 +357,9 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
 
     /* MOTD (command) */
   case SILC_COMMAND_MOTD:
-    debug ("SILC_COMMAND_MOTD");
+    debug_info ("SILC_COMMAND_MOTD");
     if (!success) {
-      debug("ERROR !success");
+      debug_info("ERROR !success");
       return;
     }
 
@@ -303,13 +370,13 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
 
     /* STATS */
   case SILC_COMMAND_STATS:
-    debug("SILC_COMMAND_STATS");
+    debug_info("SILC_COMMAND_STATS");
     if (!success) {
-      debug("ERROR !success");
+      debug_info("ERROR !success");
       return;
     }
     else {
-      debug("not implemented!");
+      debug_message("parsing stats not implemented yet");
       /* fixme: parse and print the stats reply */
     }
 
@@ -318,9 +385,9 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
 
     /* TOPIC */
   case SILC_COMMAND_TOPIC:
-    debug("SILC_COMMAND_TOPIC");
+    debug_info("SILC_COMMAND_TOPIC");
     if (!success) {
-      debug("ERROR !success");
+      debug_info("ERROR !success");
       return;
     }
 
@@ -331,10 +398,10 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
 
     /* LEAVE */
   case SILC_COMMAND_LEAVE:
-    debug("SILC_COMMAND_LEAVE");
+    debug_info("SILC_COMMAND_LEAVE");
 
     if (!success) {
-      debug("ERROR !success");
+      debug_info("ERROR !success");
       return;
     }
 
@@ -358,7 +425,7 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
 
     /* JOIN */
   case SILC_COMMAND_JOIN:
-    debug("SILC_COMMAND_JOIN");
+    debug_info("SILC_COMMAND_JOIN");
 
     debug("status code: '%d'", status);
 
@@ -393,16 +460,16 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
       (void)va_arg(va,  SilcBuffer); /* channel pubkeys */
     }
     else {
-      debug("success=FALSE");
+      debug_info("success=FALSE");
     }
 
     switch(status) {
     case SILC_STATUS_OK:
-      debug("SILC_STATUS_OK");
+      debug_info("SILC_STATUS_OK");
       break;
 
     case SILC_STATUS_ERR_USER_ON_CHANNEL:
-      debug("SILC_STATUS_ERR_USER_ON_CHANNEL");
+      debug_info("SILC_STATUS_ERR_USER_ON_CHANNEL");
       client_id = va_arg(va, SilcClientID *);
       channel_id = va_arg(va, SilcChannelID *);
       if (channel_id) {
@@ -433,18 +500,19 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
       break;
 
     case SILC_STATUS_ERR_NOT_REGISTERED:
-      debug("SILC_STATUS_ERR_NOT_REGISTERED");
+      debug_info("SILC_STATUS_ERR_NOT_REGISTERED");
       printconsole(_("You are not yet connected to server."));
       break;
 
     case SILC_STATUS_ERR_BAD_PASSWORD:
-      debug("SILC_STATUS_ERR_BAD_PASSWORD");
+      debug_info("SILC_STATUS_ERR_BAD_PASSWORD");
       printconsole(_("Password required for the channel."));
-      /* fixme: should ask the password from the user?! */
+      /* FIXME: should ask the password from the user?! */
+      debug_error("Silky doesn't yet support password protected channels.");
       break;
 
     case SILC_STATUS_ERR_NOT_INVITED:
-      debug("SILC_STATUS_ERR_NOT_INVITED");
+      debug_info("SILC_STATUS_ERR_NOT_INVITED");
       channel_id = va_arg(va, SilcChannelID *);
       if (channel_id == NULL) {
 	debug("null channel_id, this should not happen! we'll continue anyway...");
@@ -456,7 +524,7 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
       break;
 
     case SILC_STATUS_ERR_CHANNEL_IS_FULL:
-      debug("SILC_STATUS_ERR_CHANNEL_IS_FULL");
+      debug_info("SILC_STATUS_ERR_CHANNEL_IS_FULL");
       channel_id = va_arg(va, SilcChannelID *);
       if (channel_id == NULL) {
 	debug("null channel_id, this should not happen! we'll continue anyway...");
@@ -465,7 +533,7 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
       break;
 
      case SILC_STATUS_ERR_BANNED_FROM_CHANNEL:
-      debug("SILC_STATUS_ERR_BANNED_FROM_CHANNEL");
+      debug_info("SILC_STATUS_ERR_BANNED_FROM_CHANNEL");
       channel_id = va_arg(va, SilcChannelID *);
       if (channel_id == NULL) {
 	debug("null channel_id, this should not happen! we'll continue anyway...");
@@ -475,7 +543,7 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
       break;
 
       default:
-	debug("Unhandled status (%d:%s)", status, silc_get_status_message(status));
+	debug_info("Unhandled status (%d:%s)", status, silc_get_status_message(status));
 	printconsole(_("Can not join the channel."));
 	break;
     }
@@ -659,9 +727,11 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
     /* CREATE STYLES FOR CHANNEL*/
     textview = GTK_TEXT_VIEW(get_stored_channel_widget(channel_name, "textviewwidget"));
 
+    /* FIXME unused?
     if (!xml_doc(CONFIG_MAIN)) {
       debug("main xmlDoc is NULL!");
     }
+    */
 
     /* set color tags for textview widget */
     set_textview_colors(textview); /* in preferences.c */
@@ -708,7 +778,7 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
 
     /* WATCH */
   case SILC_COMMAND_WATCH:
-    debug("reply for SILC_COMMAND_WATCH");
+    debug_info("reply for SILC_COMMAND_WATCH");
 
     /* refresh the buddy list */
     refresh_gui_buddylist();
@@ -717,14 +787,15 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
       printconsole(_("OK"));
     }
     else {
-      debug(_("Add failed"));
+      debug_error("Add failed, status: '%d'", status);
       if (status == 24) { /* FIXME: what's the enum? */
-	printconsole(_("User already on list"));
+	//	printconsole(_("User already on list"));
 	break;
       }
 
       if (status == 54) { /* FIXME: what's the enum? */
-	printconsole(_("Service timed out"));
+	//	printconsole(_("Service timed out"));
+	// FIXME: should have saved the command earlier, and now we should try to resend it
 	break;
       }
 
@@ -737,10 +808,10 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
 
     /* CUMODE */
   case SILC_COMMAND_CUMODE:
-    debug("reply for SILC_COMMAND_CUMODE");
+    debug_info("reply for SILC_COMMAND_CUMODE");
 
     if (!success) {
-      debug("No success");
+      debug_info("No success");
 
       if (status == SILC_STATUS_ERR_NO_CHANNEL_FOPRIV) {
 	channel_id = va_arg(va, SilcChannelID *);
@@ -756,7 +827,7 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
 	error_no_channel_oper_priv(channel_entry->channel_name);
       }
 
-      debug("error '%d' for SILC_COMMAND_KICK", status);
+      debug_info("error '%d' for SILC_COMMAND_KICK", status);
 
     }
     if (success) {
@@ -769,7 +840,7 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
 
     /* KICK */
   case SILC_COMMAND_KICK:
-    debug("reply for SILC_COMMAND_KICK");
+    debug_info("reply for SILC_COMMAND_KICK");
 
     if (!success) {
       debug("not success");
@@ -777,7 +848,7 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
       if (status == SILC_STATUS_ERR_NO_CHANNEL_PRIV) {
 	channel_id = va_arg(va, SilcChannelID *);
 	channel_entry = silc_client_get_channel_by_id(client, conn, channel_id);
-	debug("Not channel oper on '%s'", channel_entry->channel_name);
+	debug_info("Not channel oper on '%s'", channel_entry->channel_name);
 	error_no_channel_oper_priv(channel_entry->channel_name);
 
       }
@@ -786,12 +857,12 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
       if (status == SILC_STATUS_ERR_NO_CHANNEL_FOPRIV) {
 	channel_id = va_arg(va, SilcChannelID *);
 	channel_entry = silc_client_get_channel_by_id(client, conn, channel_id);
-	debug("Not channel founder on '%s'", channel_entry->channel_name);
+	debug_info("Not channel founder on '%s'", channel_entry->channel_name);
 	error_no_channel_founder_priv(channel_entry->channel_name);
       }
 
       if (status == SILC_STATUS_ERR_USER_NOT_ON_CHANNEL) {
-	debug("User not on given channel");
+	debug_info("User not on given channel");
 	channel_id = va_arg(va, SilcChannelID *);
 	channel_entry = silc_client_get_channel_by_id(client, conn, channel_id);
 	error_user_not_on_channel(channel_entry->channel_name);
@@ -811,7 +882,7 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
 
    /* CMODE */
   case SILC_COMMAND_CMODE:
-    debug("reply for SILC_COMMAND_CMODE");
+    debug_info("reply for SILC_COMMAND_CMODE");
 
     if (!success) {
 
@@ -820,7 +891,7 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
 	channel_id = va_arg(va, SilcChannelID *);
 	channel_entry = silc_client_get_channel_by_id(client, conn, channel_id);
 	set_cmode_window_widgets(channel_entry->channel_name, channel_entry->mode); /* set modes in gui */
-	debug("Not channel oper on '%s'", channel_entry->channel_name);
+	debug_info("Not channel oper on '%s'", channel_entry->channel_name);
 	error_no_channel_oper_priv(channel_entry->channel_name);
       }
 
@@ -829,11 +900,11 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
 	channel_id = va_arg(va, SilcChannelID *);
 	channel_entry = silc_client_get_channel_by_id(client, conn, channel_id);
 	set_cmode_window_widgets(channel_entry->channel_name, channel_entry->mode); /* set modes in gui */
-	debug("Not channel founder on '%s'", channel_entry->channel_name);
+	debug_info("Not channel founder on '%s'", channel_entry->channel_name);
 	error_no_channel_founder_priv(channel_entry->channel_name);
       }
 
-      debug("error '%d' for SILC_COMMAND_CMODE", status);
+      debug_info("error '%d' for SILC_COMMAND_CMODE", status);
 
 
     }
@@ -841,7 +912,7 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
 
     /* LIST */
   case SILC_COMMAND_LIST:
-    debug("reply for SILC_COMMAND_LIST");
+    debug_info("reply for SILC_COMMAND_LIST");
 
     if (success) {
       channel_entry = va_arg(va, SilcChannelEntry);
@@ -855,7 +926,17 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
       }
 
 
-      debug("LIST for %s. %d users. Topic: '%s'", channel_name, list_count, channel_topic);
+
+
+      debug_info("LIST for %s. %d users. Topic: '%s'", channel_name, list_count, channel_topic);
+      if (channel_topic && !g_utf8_validate(channel_topic, -1, NULL)) {
+	debug("*** INVALID utf8 in channel topic, skipping");
+	channel_topic = strdup("non-utf8 topic, can't show");
+      }
+      else if (channel_topic==NULL)
+      {
+		  channel_topic= g_strdup_printf("");
+  }
       printconsole(g_strdup_printf("%s (%d) %s", channel_name, (unsigned int) list_count, channel_topic));
 
       /* this needs to be last... */
@@ -873,8 +954,7 @@ void silc_command_reply(SilcClient client, SilcClientConnection conn,
     break;
 
   case SILC_COMMAND_PING: /* 12 */
-    debug("reply for SILC_COMMAND_PING");
-    lastpong = time(NULL);
+    debug_info("reply for SILC_COMMAND_PING");
     break;
 
 
