@@ -22,10 +22,22 @@
 
 */
 
+#include <glib.h>
+
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
 
-#include "includes.h"
 #include "ui_buddylist.h"
 #include "ui_channel.h"
+#include "ui_console.h"
+#include "common.h"
+#include "support.h"
+#include "gui.h"
+#include "silc-notify.h"
+#include "log.h"
+#include "channel.h"
+#include "callbacks.h"
 
 extern silkyStruct *silky;
 extern GladeXML *xmlmain;
@@ -37,6 +49,8 @@ extern gchar * glade_silky_channel_label;
 
 extern SilkyChannel *channels;
 
+extern gint lastpong;
+
 void silc_notify(SilcClient client, SilcClientConnection conn,
                         SilcNotifyType type, ...)
 {
@@ -49,6 +63,7 @@ void silc_notify(SilcClient client, SilcClientConnection conn,
   SilcStatus status;
   char *str;
   gchar *str_clientid;
+  gchar *str_fingerprint;
   GtkNotebook *storednotebook;
   gchar *str_client_id;
   gchar *str_client_id2;
@@ -72,6 +87,8 @@ void silc_notify(SilcClient client, SilcClientConnection conn,
   debug("silc_notify()");
   va_start(va, type);
 
+  lastpong = time(NULL); /* update date of last received data, used in timer.c */
+
   /*
     This is the part where we receive all kind of different data from the
     server. silky is only interested in receiving the less important
@@ -80,26 +97,26 @@ void silc_notify(SilcClient client, SilcClientConnection conn,
   switch(type) {
 
   case SILC_NOTIFY_TYPE_NONE:
-    debug("SILC_NOTIFY_TYPE_NONE");
+    debug_info("SILC_NOTIFY_TYPE_NONE");
     str = va_arg(va, char *);
     printconsole(g_strdup_printf("%s", str));
     break;
 
   case SILC_NOTIFY_TYPE_MOTD:
-    debug("SILC_NOTIFY_TYPE_MOTD");
+    debug_info("SILC_NOTIFY_TYPE_MOTD");
     str = va_arg(va, char *);
     printconsole(g_strdup_printf("%s", str));
     break;
 
   case SILC_NOTIFY_TYPE_ERROR:
     status = va_arg(va, int );
-    debug("SILC_NOTIFY_TYPE_ERROR (%d)", status);
+    debug_info("SILC_NOTIFY_TYPE_ERROR (%d)", status);
 
     switch(status) {
 
       /* After sending a private message; There was no such user */
     case SILC_STATUS_ERR_NO_SUCH_CLIENT_ID:
-      debug("No such client id");
+      debug_info("No such client id");
       client_id = va_arg(va, SilcClientID *);
       if (!client_id) {
 	debug("Did not get client_id in _NO_SUCH_CLIENT_ID");
@@ -113,7 +130,7 @@ void silc_notify(SilcClient client, SilcClientConnection conn,
 	debug("Did not find client_entry");
       }
       else {
-	debug("\tnick: '%s'", client_entry->nickname);
+	debug_info("\tnick: '%s'", client_entry->nickname);
       }
 
       printconsole(_("No such user."));
@@ -130,7 +147,7 @@ void silc_notify(SilcClient client, SilcClientConnection conn,
 
 
   case SILC_NOTIFY_TYPE_KICKED:
-    debug("SILC_NOTIFY_TYPE_KICKED");
+    debug_info("SILC_NOTIFY_TYPE_KICKED");
     client_entry = va_arg(va, SilcClientEntry);
     str = va_arg(va, char *);
     client_entry2 = va_arg(va, SilcClientEntry);
@@ -138,7 +155,7 @@ void silc_notify(SilcClient client, SilcClientConnection conn,
 
     if (SILC_ID_COMPARE(client_entry->id, conn->local_entry->id, sizeof(SilcClientID) ) ) {
       /* I GOT KICKED */
-      debug("I have been kicked out from the channel '%s'", channel_entry->channel_name);
+      debug_info("I have been kicked out from the channel '%s'", channel_entry->channel_name);
 
       /* Close the tab in UI */
       storednotebook = GTK_NOTEBOOK(glade_xml_get_widget (xmlmain, "tabs"));
@@ -175,11 +192,11 @@ void silc_notify(SilcClient client, SilcClientConnection conn,
     break;
 
   case SILC_NOTIFY_TYPE_JOIN:
-    debug("SILC_NOTIFY_TYPE_JOIN");
+    debug_info("SILC_NOTIFY_TYPE_JOIN");
 
     client_entry = va_arg(va, SilcClientEntry);
     channel_entry = va_arg(va, SilcChannelEntry);
-    debug("'%s' (%s@%s) joined '%s'",  client_entry->nickname, client_entry->username, client_entry->hostname, channel_entry->channel_name);
+    debug_info("'%s' (%s@%s) joined '%s'",  client_entry->nickname, client_entry->username, client_entry->hostname, channel_entry->channel_name);
     printchannel(channel_entry->channel_name, g_strdup_printf(_("%s (%s@%s) has joined %s"),  client_entry->nickname, client_entry->username, client_entry->hostname, channel_entry->channel_name));
     /* not yet fully implemented, disabled for now
     silky_print(MSG_MOTION | MSG_PUBLIC, channel_entry, "%s joined %s", client_entry->nickname, channel_entry->channel_name);
@@ -201,11 +218,11 @@ void silc_notify(SilcClient client, SilcClientConnection conn,
     break;
 
   case SILC_NOTIFY_TYPE_LEAVE:
-    debug("SILC_NOTIFY_TYPE_LEAVE");
+    debug_info("SILC_NOTIFY_TYPE_LEAVE");
     client_entry = va_arg(va, SilcClientEntry);
     channel_entry = va_arg(va, SilcChannelEntry);
 
-    debug("'%s' (%s@%s) left '%s'",  client_entry->nickname, client_entry->username, client_entry->hostname, channel_entry->channel_name);
+    debug_info("'%s' (%s@%s) left '%s'",  client_entry->nickname, client_entry->username, client_entry->hostname, channel_entry->channel_name);
     printchannel(channel_entry->channel_name, g_strdup_printf(_("%s (%s@%s) left %s"), client_entry->nickname, client_entry->username, client_entry->hostname, channel_entry->channel_name));
 
     debug("freeing channel member from list...");
@@ -225,7 +242,7 @@ void silc_notify(SilcClient client, SilcClientConnection conn,
 	str_client_id = silc_id_id2str(client_entry->id, SILC_ID_CLIENT);
 	str_client_id2 = silc_id_id2str(client_entry2->id, SILC_ID_CLIENT);
 
-    debug("NICK_CHANGE: '%s' is now known as '%s'",
+    debug_info("NICK_CHANGE: '%s' is now known as '%s'",
 	  client_entry->nickname, client_entry2->nickname);
 
 
@@ -280,7 +297,7 @@ void silc_notify(SilcClient client, SilcClientConnection conn,
 
 
   case SILC_NOTIFY_TYPE_TOPIC_SET:
-    debug("SILC_NOTIFY_TYPE_TOPIC_SET");
+    debug_info("SILC_NOTIFY_TYPE_TOPIC_SET");
     /* SilcIdType setter_id_type, void *setter_entry, char *topic, SilcChannelEntry channel */
 
    idtype = va_arg(va, int);
@@ -298,9 +315,12 @@ void silc_notify(SilcClient client, SilcClientConnection conn,
 
      channel_topic_set_by_entry( channel_entry, str );
 
+     debug_info(channel_entry->channel_name, g_strdup_printf(_("%s (%s@%s) changed topic for channel %s to '%s'"),
+		       client_entry->nickname, client_entry->username, client_entry->hostname, channel_entry->channel_name, str));
      printchannel(channel_entry->channel_name, g_strdup_printf(_("%s (%s@%s) changed topic for channel %s to '%s'"),
 		  client_entry->nickname, client_entry->username, client_entry->hostname, channel_entry->channel_name, str));
-   }
+
+  }
    else {
      printchannel(channel_entry->channel_name, _("UNHANDLED TOPIC SET EVENT: Please report a bug to Silky development team"));
    }
@@ -310,7 +330,7 @@ void silc_notify(SilcClient client, SilcClientConnection conn,
 
 
   case SILC_NOTIFY_TYPE_CMODE_CHANGE:
-    debug("SILC_NOTIFY_TYPE_CMODE_CHANGE:");
+    debug_info("SILC_NOTIFY_TYPE_CMODE_CHANGE:");
 
     /* get attributes from va_list */
     idtype = va_arg(va, int); /* changer's id */
@@ -353,13 +373,13 @@ void silc_notify(SilcClient client, SilcClientConnection conn,
 
     silky_gui_check_channel_cmodes(channel_entry, str_mode);
 
-    debug("*** '%s' changed mode '%s' on '%s'", changer, str_mode, channel_entry->channel_name);
+    debug_info("*** '%s' changed mode '%s' on '%s'", changer, str_mode, channel_entry->channel_name);
 
     break;
 
 
   case SILC_NOTIFY_TYPE_CUMODE_CHANGE:
-    debug("SILC_NOTIFY_TYPE_CUMODE_CHANGE:");
+    debug_info("SILC_NOTIFY_TYPE_CUMODE_CHANGE:");
 
     idtype = va_arg(va, int);
     entry = va_arg(va, void *);
@@ -460,7 +480,7 @@ void silc_notify(SilcClient client, SilcClientConnection conn,
       str_mode = g_strconcat(str_mode, "q", NULL);
     }
 
-    debug("*** '%s' changed mode '%s' for '%s' on '%s'", changer, str_mode, client_entry->nickname, channel_entry->channel_name);
+    debug_info("*** '%s' changed mode '%s' for '%s' on '%s'", changer, str_mode, client_entry->nickname, channel_entry->channel_name);
     printchannel(channel_entry->channel_name, g_strdup_printf(_("%s changed mode for '%s' on '%s' to %s"),
 		 changer, client_entry->nickname, channel_entry->channel_name, str_mode));
     break;
@@ -468,21 +488,34 @@ void silc_notify(SilcClient client, SilcClientConnection conn,
 
     /* INVITE: SilcClientChannel channel, char *channel_name, SilcClientEntry inviter */
   case SILC_NOTIFY_TYPE_INVITE:
-    debug("SILC_NOTIFY_TYPE_INVITE:");
+    debug_info("SILC_NOTIFY_TYPE_INVITE:");
     channel_entry = va_arg(va, SilcChannelEntry);
     str = va_arg(va, char *);
     client_entry = va_arg(va, SilcClientEntry);
 
-
-    debug("I was invited to channel '%s' by '%s (%s@%s)", str, client_entry->nickname, client_entry->username, client_entry->hostname);
+    debug_info("I was invited to channel '%s' by '%s (%s@%s)", str, client_entry->nickname, client_entry->username, client_entry->hostname);
     printconsole(g_strdup_printf(_("%s (%s@%s) invites you to channel %s."), client_entry->nickname, client_entry->username, client_entry->hostname, str));
 
-    /* FIXME: perhaps could ask user if he wants to join? */
+    gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xmlmain, "label_recinv_topic")),
+	g_strdup_printf("<b>Join channel '%s'?</b>", str));
+
+    gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xmlmain, "label_recinv_text")),
+	g_strdup_printf(_("You were invited by %s\n(%s@%s)"),
+	client_entry->nickname, client_entry->username, client_entry->hostname));
+
+    gtk_widget_show(glade_xml_get_widget(xmlmain, "dialog_received_invite"));
+
+    g_signal_connect(glade_xml_get_widget(xmlmain, "button_received_invite_show_user_info"), "activate", G_CALLBACK(on_received_invite_user_info), client_entry);
+    g_signal_connect(glade_xml_get_widget(xmlmain, "button_received_invite_show_user_info"), "released", G_CALLBACK(on_received_invite_user_info), client_entry);
+
+    /* store channelname, so dialog 'Yes' callback function can access it */
+    g_object_set_data(G_OBJECT(glade_xml_get_widget(xmlmain, "dialog_received_invite")), "invited_channel", g_strdup(str));
+
     break;
 
     /* SIGNOFF */
   case SILC_NOTIFY_TYPE_SIGNOFF:
-    debug("SILC_NOTIFY_TYPE_SIGNOFF:");
+    debug_info("SILC_NOTIFY_TYPE_SIGNOFF:");
 
     client_entry = va_arg(va, SilcClientEntry); /* user */
     str = va_arg(va, char *); /* reason */
@@ -491,7 +524,7 @@ void silc_notify(SilcClient client, SilcClientConnection conn,
       debug("Did not get client_entry");
       return;
     }
-    debug("User: '%s' quitted with reason '%s'", client_entry->nickname, str);
+    debug_info("User: '%s' quitted with reason '%s'", client_entry->nickname, str);
 
 
     debug("Starting iteration of userchannels");
@@ -526,7 +559,7 @@ void silc_notify(SilcClient client, SilcClientConnection conn,
       debug("Iteration done.");
     }
     else {
-      debug("Got NULL client_entry->channels. This is a bug in client library.");
+      debug_message("Got NULL client_entry->channels. This must be a bug in the SILC Client library.");
     }
 
 
@@ -536,28 +569,33 @@ void silc_notify(SilcClient client, SilcClientConnection conn,
     /* WATCH */
   case SILC_NOTIFY_TYPE_WATCH:
     /* SilcClientEntry watched_client, char *new_nickname, SilcUInt32 user_mode, SilcNotifyType notification */
-    debug("SILC_NOTIFY_TYPE_WATCH");
+    debug_info("SILC_NOTIFY_TYPE_WATCH");
 
     client_entry = va_arg(va, SilcClientEntry);
     str = va_arg(va, char *);
     mode = va_arg(va, SilcUInt32);
     notification = va_arg(va, int);
 
+    str_fingerprint = silc_fingerprint(client_entry->fingerprint, client_entry->fingerprint_len);
+    buddy_set_clientid_by_fp(str_fingerprint, client_entry->id);
+
     switch (notification) {
     case SILC_NOTIFY_TYPE_NICK_CHANGE:
-      debug("SILC_NOTIFY_TYPE_NICK_CHANGE");
-      debug("WATCH '%s' is now known as '%s'", client_entry->nickname, str);
-      if (buddy_set_nick_by_fp(silc_fingerprint(client_entry->fingerprint, client_entry->fingerprint_len), str)) {
-	debug("nick changed in the buddy list");
+
+      debug_info("SILC_NOTIFY_TYPE_NICK_CHANGE");
+      debug_info("WATCH '%s' is now known as '%s'", str_fingerprint, client_entry->nickname);
+
+      if (buddy_set_nick_by_fp(str_fingerprint, client_entry->nickname)) {
  	refresh_gui_buddylist();
-     } else {
-	debug("nick not found in the buddy list");
+      } else {
+	debug("fingerprint not found from the buddy list");
       }
+
       break;
 
     case SILC_NOTIFY_TYPE_UMODE_CHANGE:
-      debug("SILC_NOTIFY_TYPE_UMODE_CHANGE");
-      debug("WATCH '%s' has changed umode %d", client_entry->nickname, mode);
+      debug_info("SILC_NOTIFY_TYPE_UMODE_CHANGE");
+      debug_info("WATCH '%s' has changed umode %d", client_entry->nickname, mode);
       if (buddy_set_mode_by_fp(silc_fingerprint(client_entry->fingerprint, client_entry->fingerprint_len), mode)) {
 	debug("mode set in the buddy list");
 	refresh_gui_buddylist();
@@ -569,7 +607,7 @@ void silc_notify(SilcClient client, SilcClientConnection conn,
       break;
 
     default:
-      debug("unhandled notification type: %d", notification);
+      debug_info("unhandled notification type: %d", notification);
       break;
 
     }
@@ -579,7 +617,7 @@ void silc_notify(SilcClient client, SilcClientConnection conn,
 
    /* UNHANDLED NOTIFIES */
   default:
-    debug("unhandled SILC_NOTIFY_TYPE (%d)", type);
+    debug_message("unhandled SILC_NOTIFY_TYPE (%d)", type);
     break;
   }
 
