@@ -1,6 +1,6 @@
 /*
     Silky - A GTK+ client for SILC.
-    Copyright (C) 2003,2004 Toni Willberg
+    Copyright (C) 2003, 2004, 2005 Toni Willberg
 
     - Callbacks for SILC protocol events
 
@@ -22,9 +22,29 @@
 
 */
 
-#include "includes.h"
-#include "ui_buddylist.h"
+#include <silcincludes.h>
+#include <silcclient.h>
+#include <glib.h>
+#include <gtk/gtk.h>
+#include <glade/glade.h>
+
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include "files.h"
+#include "support.h"
+#include "keys.h"
+#include "log.h"
+#include "gui.h"
+#include "silc-command-reply.h"
+#include "silc-notify.h"
+
+#include "ui_buddylist.h"
+#include "ui_console.h"
+#include "ui_channel.h"
+#include "ui_query.h"
+#include "common.h"
 
 extern GladeXML *xmlmain;
 extern silkyStruct *silky;   /* the client */
@@ -85,7 +105,7 @@ void silc_channel_message(SilcClient client, SilcClientConnection conn,
   SilcUInt32 mime_data_len;
   bool valid_mime;
   unsigned int header_length;
-  bool error;
+  bool error, sgn = 0;
   //  char *error_message = "<Unknown mime type>";
 
   debug("silc_channel_message()");
@@ -98,6 +118,11 @@ void silc_channel_message(SilcClient client, SilcClientConnection conn,
 			       transfer_encoding, sizeof(transfer_encoding) - 1,
 			       &mime_data_buffer, &mime_data_len);
 
+  if( flags & SILC_MESSAGE_FLAG_SIGNED ) {
+    SilcMessageSignedPayload sig = silc_message_get_signature(payload);
+    sgn = verify_message_signature(sender, sig, payload);
+  }
+
   if (valid_mime == TRUE) {
     debug("MIME: A valid MIME message received, ");
     debug("MIME: type: '%s', transfer encoding: '%s' size: '%d'", content_type, transfer_encoding,  (unsigned int) mime_data_len);
@@ -116,7 +141,7 @@ void silc_channel_message(SilcClient client, SilcClientConnection conn,
 	!strcmp(content_type, "image/jpeg") ||
 	!strcmp(content_type, "text/plain")
 	) {
-      print_mime_to_channel(sender, channel->channel_name, g_strdup(content_type), mime_data_buffer, mime_data_len);
+      print_mime_to_channel(sender, channel->channel_name, 0, g_strdup(content_type), mime_data_buffer, mime_data_len, sgn);
       error = FALSE;
     }
 
@@ -132,7 +157,7 @@ void silc_channel_message(SilcClient client, SilcClientConnection conn,
   else {
     debug("NOT MIME, a plain text message");
     /* PLAIN TEXT or INVALID MIME */
-      print_mime_to_channel(sender, channel->channel_name, g_strdup("text/plain"), (int)flags, message);
+      print_mime_to_channel(sender, channel->channel_name, sgn, g_strdup("text/plain"), (int)flags, message);
   }
   debug("silc_channel_message() returning ");
 
@@ -285,7 +310,7 @@ void silc_connected(SilcClient client, SilcClientConnection conn,
   silky->state = STATE_CONNECTED;
   silky->conn = conn;
 
-  /* init watchlist */
+  /* init buddylist WATCH commands, buddylist.c */
   buddy_init_watchlist();
 
   /* enable disconnect from the menu */
@@ -325,9 +350,6 @@ void silc_disconnected(SilcClient client, SilcClientConnection conn,
   GtkWidget *statusbar;
   guint context;
 
-  GtkWidget *page;
-  GtkListStore *stored_liststore;
-
   debug("setting state to STATE_DISCONNECTED");
   silky->state = STATE_DISCONNECTED;
 
@@ -360,7 +382,7 @@ void silc_disconnected(SilcClient client, SilcClientConnection conn,
   for( ch = channels; ch; ch = ch->next ) {
     /* set nicklist insensitive */
     GtkTreeView *treeview = GTK_TREE_VIEW(glade_xml_get_widget(ch->glade_xml, "nicktreeview"));
-    gtk_widget_set_sensitive(treeview, FALSE);
+    gtk_widget_set_sensitive( GTK_WIDGET(treeview), FALSE );
   }
 
   /* free all channel lists */
@@ -443,33 +465,36 @@ void silc_verify_public_key(SilcClient client, SilcClientConnection conn,
                                    SilcUInt32 pk_len, SilcSKEPKType pk_type,
                                    SilcVerifyPublicKey completion, void *ctx) {
 
-	SilcPublicKey pkey;
+  SilcPublicKey pkey;
   gboolean result;
-	bool ret;
-	char pkdir[256];
+  bool ret;
 
   debug("silc_verify_public_key()");
 
-	ret = silc_pkcs_public_key_decode(pk, pk_len, &pkey);
-	if (ret != TRUE) { /* That was not a SILC key! */
-	    debug("could not decode received SILC public key");
-	    printconsole(_("Invalid key."));
-	    completion(FALSE, ctx);
-	    return;
-	}
+  ret = silc_pkcs_public_key_decode(pk, pk_len, &pkey);
+  if (ret != TRUE) { /* That was not a SILC key! */
+    debug("could not decode received SILC public key");
+    printconsole(_("Invalid key."));
+    completion(FALSE, ctx);
+    return;
+  }
 
-	/* If the key already exists, skip saving it entirely */
-	memset(pkdir, 0, 256);
-	snprintf(pkdir, 256, "%s%cserverkeys", silkydir, DIR_SEPARATOR);
-	result = silky_check_dir_for_key(pkdir, pkey);
-	if (result) {
-	    debug("received SILC public key was already stored");
-	    completion(TRUE, ctx);
-	    return;
-	}
+  /* If the key already exists, skip saving it entirely */
+  if (conn_type == SILC_SOCKET_TYPE_SERVER || conn_type == SILC_SOCKET_TYPE_ROUTER) {
+    result = silky_known_server_key(pkey);
+  } else {
+    result = silky_known_client_key(pkey);
+  }
+
+
+  if (result) {
+    debug("received SILC public key was already stored");
+    completion(TRUE, ctx);
+    return;
+  }
 
     /* Save the key */
-  result = silc_save_public_key(client, conn, NULL, conn_type, pk, pk_len,
+  result = silky_save_public_key(client, conn, NULL, conn_type, pk, pk_len,
 				pk_type, completion, ctx);
   if (result == TRUE) {
     debug("user accepted the key");
@@ -483,7 +508,7 @@ void silc_verify_public_key(SilcClient client, SilcClientConnection conn,
   }
 
   debug("returning");
-	return;
+  return;
 }
 
 
