@@ -1,3 +1,9 @@
+#ifndef XMLCONFIG_H__
+#define XMLCONFIG_H__
+
+#include <libxml/parser.h>
+#include <libxml/xpath.h>
+
 xmlNodePtr xml_create_node(gint config_type, gint node_cn);
 xmlNodePtr xml_get_node(gint config_type, xmlDocPtr doc, gint node_cn);
 
@@ -13,3 +19,5 @@ gint xml_find_option_num_by_name(gint config_type, xmlChar *name);
 gint xml_write_config(gint config_type, gchar *filename);
 
 xmlDocPtr xml_doc(gint type);
+
+#endif
